@@ -13,7 +13,10 @@ module.exports = {
       'BUILDING',
       'VS15',
       'CHANGELOG',
+      'CONTRIBUTING',
       'PRIMER',
+      'IQ_FORMATS',
+      'ANALYZE',
       'OPERATION',
       'DATA_FORMAT',
       'HARDWARE',
