@@ -102,8 +102,6 @@ if __name__ == '__main__':
     gpsp = GpsPoller()
 
     sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
-    # allow multiple sockets to use the same PORT number
-    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
     sock.bind((UDP_IP, UDP_PORT))
 
     try:
