@@ -55,6 +55,10 @@ Getting rtl_433 devices back after Home Assistant restarts will happen
 more quickly if MQTT retain is enabled. Note however that definitions
 for any transitient devices/false positives will retained indefinitely.
 
+If your sensor values change infrequently and you prefer to write the most
+recent value even if not changed set -f to append "force_update = true" to
+all configs. This is useful if you're graphing the sensor data or want to
+alert on missing data.
 
 Suggestions:
 
@@ -94,6 +98,7 @@ There is a single global set of field mappings to Home Assistant meta data.
 
 import os
 import argparse
+import logging
 import time
 import json
 import paho.mqtt.client as mqtt
@@ -122,7 +127,8 @@ mappings = {
             "device_class": "temperature",
             "name": "Temperature",
             "unit_of_measurement": "°C",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
     "temperature_1_C": {
@@ -132,7 +138,8 @@ mappings = {
             "device_class": "temperature",
             "name": "Temperature 1",
             "unit_of_measurement": "°C",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
     "temperature_2_C": {
@@ -142,7 +149,8 @@ mappings = {
             "device_class": "temperature",
             "name": "Temperature 2",
             "unit_of_measurement": "°C",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
     "temperature_F": {
@@ -152,7 +160,8 @@ mappings = {
             "device_class": "temperature",
             "name": "Temperature",
             "unit_of_measurement": "°F",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -163,7 +172,8 @@ mappings = {
             "device_class": "battery",
             "name": "Battery",
             "unit_of_measurement": "%",
-            "value_template": "{{ float(value|int) * 99 + 1 }}"
+            "value_template": "{{ float(value) * 99 + 1 }}",
+            "state_class": "measurement"
         }
     },
 
@@ -174,7 +184,8 @@ mappings = {
             "device_class": "humidity",
             "name": "Humidity",
             "unit_of_measurement": "%",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -182,10 +193,11 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "H",
         "config": {
-            "device_class": "moisture",
+            "device_class": "humidity",
             "name": "Moisture",
             "unit_of_measurement": "%",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -196,7 +208,20 @@ mappings = {
             "device_class": "pressure",
             "name": "Pressure",
             "unit_of_measurement": "hPa",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
+        }
+    },
+
+    "pressure_kPa": {
+        "device_type": "sensor",
+        "object_suffix": "P",
+        "config": {
+            "device_class": "pressure",
+            "name": "Pressure",
+            "unit_of_measurement": "kPa",
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -204,10 +229,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
-            "device_class": "weather",
             "name": "Wind Speed",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -218,7 +243,8 @@ mappings = {
             "device_class": "weather",
             "name": "Wind Speed",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -229,7 +255,8 @@ mappings = {
             "device_class": "weather",
             "name": "Wind Speed",
             "unit_of_measurement": "mi/h",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -239,7 +266,8 @@ mappings = {
         "config": {
             "name": "Wind Average",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ float(value|float) * 3.6 | round(2) }}"
+            "value_template": "{{ (float(value|float) * 3.6) | round(2) }}",
+            "state_class": "measurement"
         }
     },
 
@@ -247,10 +275,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WS",
         "config": {
-            "device_class": "weather",
             "name": "Wind Speed",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ float(value|float) * 3.6 }}"
+            "value_template": "{{ float(value|float) * 3.6 }}",
+            "state_class": "measurement"
         }
     },
 
@@ -258,10 +286,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "GS",
         "config": {
-            "device_class": "weather",
             "name": "Gust Speed",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -271,7 +299,8 @@ mappings = {
         "config": {
             "name": "Wind max",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ float(value|float) * 3.6 | round(2) }}"
+            "value_template": "{{ (float(value|float) * 3.6) | round(2) }}",
+            "state_class": "measurement"
         }
     },
 
@@ -279,10 +308,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "GS",
         "config": {
-            "device_class": "weather",
             "name": "Gust Speed",
             "unit_of_measurement": "km/h",
-            "value_template": "{{ float(value|float) * 3.6 }}"
+            "value_template": "{{ float(value|float) * 3.6 }}",
+            "state_class": "measurement"
         }
     },
 
@@ -290,10 +319,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "WD",
         "config": {
-            "device_class": "weather",
             "name": "Wind Direction",
             "unit_of_measurement": "°",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -301,10 +330,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "RT",
         "config": {
-            "device_class": "weather",
             "name": "Rain Total",
             "unit_of_measurement": "mm",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "total_increasing"
         }
     },
 
@@ -312,10 +341,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "RR",
         "config": {
-            "device_class": "weather",
             "name": "Rain Rate",
             "unit_of_measurement": "mm/h",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -325,7 +354,8 @@ mappings = {
         "config": {
             "name": "Rain Total",
             "unit_of_measurement": "mm",
-            "value_template": "{{ float(value|float) * 25.4 | round(2) }}"
+            "value_template": "{{ (float(value|float) * 25.4) | round(2) }}",
+            "state_class": "total_increasing"
         }
     },
 
@@ -335,7 +365,8 @@ mappings = {
         "config": {
             "name": "Rain Rate",
             "unit_of_measurement": "mm/h",
-            "value_template": "{{ float(value|float) * 25.4 | round(2) }}"
+            "value_template": "{{ (float(value|float) * 25.4) | round(2) }}",
+            "state_class": "measurement"
         }
     },
 
@@ -367,7 +398,8 @@ mappings = {
         "config": {
             "device_class": "signal_strength",
             "unit_of_measurement": "dB",
-            "value_template": "{{ value|float|round(2) }}"
+            "value_template": "{{ value|float|round(2) }}",
+            "state_class": "measurement"
         }
     },
 
@@ -377,7 +409,8 @@ mappings = {
         "config": {
             "device_class": "signal_strength",
             "unit_of_measurement": "dB",
-            "value_template": "{{ value|float|round(2) }}"
+            "value_template": "{{ value|float|round(2) }}",
+            "state_class": "measurement"
         }
     },
 
@@ -387,7 +420,8 @@ mappings = {
         "config": {
             "device_class": "signal_strength",
             "unit_of_measurement": "dB",
-            "value_template": "{{ valuei|float|round(2) }}"
+            "value_template": "{{ value|float|round(2) }}",
+            "state_class": "measurement"
         }
     },
 
@@ -395,10 +429,10 @@ mappings = {
         "device_type": "sensor",
         "object_suffix": "D",
         "config": {
-            "device_class": "depth",
             "name": "Depth",
             "unit_of_measurement": "cm",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -409,7 +443,8 @@ mappings = {
             "device_class": "power",
             "name": "Power",
             "unit_of_measurement": "W",
-            "value_template": "{{ value|float }}"
+            "value_template": "{{ value|float }}",
+            "state_class": "measurement"
         }
     },
 
@@ -420,7 +455,8 @@ mappings = {
             "device_class": "weather",
             "name": "Outside Luminancee",
             "unit_of_measurement": "lux",
-            "value_template": "{{ value|int }}"
+            "value_template": "{{ value|int }}",
+            "state_class": "measurement"
         }
     },
 
@@ -431,7 +467,8 @@ mappings = {
             "device_class": "weather",
             "name": "UV Index",
             "unit_of_measurement": "UV Index",
-            "value_template": "{{ value|int }}"
+            "value_template": "{{ value|int }}",
+            "state_class": "measurement"
         }
     },
 
@@ -441,7 +478,8 @@ mappings = {
         "config": {
             "name": "Lightning Distance",
             "unit_of_measurement": "mi",
-            "value_template": "{{ value|int }}"
+            "value_template": "{{ value|int }}",
+            "state_class": "measurement"
         }
     },
 
@@ -451,7 +489,8 @@ mappings = {
         "config": {
             "name": "Lightning Distance",
             "unit_of_measurement": "mi",
-            "value_template": "{{ value|int }}"
+            "value_template": "{{ value|int }}",
+            "state_class": "measurement"
         }
     },
 
@@ -460,7 +499,8 @@ mappings = {
         "object_suffix": "strcnt",
         "config": {
             "name": "Lightning Strike Count",
-            "value_template": "{{ value|int }}"
+            "value_template": "{{ value|int }}",
+            "state_class": "total_increasing"
         }
     },
 }
@@ -469,26 +509,29 @@ mappings = {
 def mqtt_connect(client, userdata, flags, rc):
     """Callback for MQTT connects."""
 
-    print("MQTT connected: " + mqtt.connack_string(rc))
+    logging.info("MQTT connected: " + mqtt.connack_string(rc))
     if rc != 0:
-        print("Could not connect. Error: " + str(rc))
+        logging.error("Could not connect. Error: " + str(rc))
     else:
+        logging.info("Subscribing to: " + args.rtl_topic)
         client.subscribe(args.rtl_topic)
 
 
 def mqtt_disconnect(client, userdata, rc):
     """Callback for MQTT disconnects."""
-    print("MQTT disconnected: " + mqtt.connack_string(rc))
+    logging.info("MQTT disconnected: " + mqtt.connack_string(rc))
 
 
 def mqtt_message(client, userdata, msg):
     """Callback for MQTT message PUBLISH."""
+    logging.debug("MQTT message: " + json.dumps(msg.payload.decode()))
+
     try:
         # Decode JSON payload
         data = json.loads(msg.payload.decode())
 
     except json.decoder.JSONDecodeError:
-        print("JSON decode error: " + msg.payload.decode())
+        logging.error("JSON decode error: " + msg.payload.decode())
         return
 
     topicprefix = "/".join(msg.topic.split("/", 2)[:2])
@@ -532,6 +575,7 @@ def publish_config(mqttc, topic, model, instance, mapping):
     now = time.time()
     if path in discovery_timeouts:
         if discovery_timeouts[path] > now:
+            logging.debug("Discovery timeout in the future for: " + path)
             return False
 
     discovery_timeouts[path] = now + args.discovery_interval
@@ -542,10 +586,12 @@ def publish_config(mqttc, topic, model, instance, mapping):
     config["unique_id"] = object_name
     config["device"] = { "identifiers": object_id, "name": object_id, "model": model, "manufacturer": "rtl_433" }
 
-    if args.debug:
-        print(path,":",json.dumps(config))
+    if args.force_update:
+        config["force_update"] = "true"
 
-    mqttc.publish(path, json.dumps(config), args.retain)
+    logging.debug(path + ":" + json.dumps(config))
+
+    mqttc.publish(path, json.dumps(config), retain=args.retain)
 
     return True
 
@@ -554,6 +600,7 @@ def bridge_event_to_hass(mqttc, topicprefix, data):
 
     if "model" not in data:
         # not a device event
+        logging.debug("Model is not defined. Not sending event to Home Assistant.")
         return
 
     model = sanitize(data["model"])
@@ -564,8 +611,7 @@ def bridge_event_to_hass(mqttc, topicprefix, data):
     instance = rtl_433_device_topic(data)
     if not instance:
         # no unique device identifier
-        if not args.quiet:
-            print("No suitable identifier found for model: ", model)
+        logging.warning("No suitable identifier found for model: ", model)
         return
 
     # detect known attributes
@@ -579,23 +625,28 @@ def bridge_event_to_hass(mqttc, topicprefix, data):
             if key not in SKIP_KEYS:
                 skipped_keys.append(key)
 
-    if published_keys and not args.quiet:
-        print("Published %s: %s" % (instance, ", ".join(published_keys)))
+    if published_keys:
+        logging.info("Published %s: %s" % (instance, ", ".join(published_keys)))
 
-        if skipped_keys and not args.quiet:
-            print("Skipped %s: %s" % (instance, ", ".join(skipped_keys)))
+        if skipped_keys:
+            logging.info("Skipped %s: %s" % (instance, ", ".join(skipped_keys)))
 
 
 def rtl_433_bridge():
     """Run a MQTT Home Assistant auto discovery bridge for rtl_433."""
 
     mqttc = mqtt.Client()
+
+    if args.debug:
+        mqttc.enable_logger()
+
     if args.user is not None:
         mqttc.username_pw_set(args.user, args.password)
     mqttc.on_connect = mqtt_connect
     mqttc.on_disconnect = mqtt_disconnect
     mqttc.on_message = mqtt_message
     mqttc.connect_async(args.host, args.port, 60)
+    logging.debug("MQTT Client: Starting Loop")
     mqttc.loop_start()
 
     while True:
@@ -613,6 +664,7 @@ def run():
 
 
 if __name__ == "__main__":
+    logging.getLogger().setLevel(logging.INFO)
 
     parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                      description=AP_DESCRIPTION,
@@ -627,6 +679,8 @@ if __name__ == "__main__":
     parser.add_argument("-p", "--port", type=int, default=1883,
                         help="MQTT port (default: %(default)s)")
     parser.add_argument("-r", "--retain", action="store_true")
+    parser.add_argument("-f", "--force_update", action="store_true",
+                        help="Append 'force_update = true' to all configs.")
     parser.add_argument("-R", "--rtl-topic", type=str,
                         default="rtl_433/+/events",
                         dest="rtl_topic",
@@ -641,6 +695,16 @@ if __name__ == "__main__":
                         help="Interval to republish config topics in seconds (default: %(default)d)")
     args = parser.parse_args()
 
+    if args.debug and args.quiet:
+        logging.critical("Debug and quiet can not be specified at the same time")
+        exit(1)
+
+    if args.debug:
+        logging.info("Enabling debug logging")
+        logging.getLogger().setLevel(logging.DEBUG)
+    if args.quiet:
+        logging.getLogger().setLevel(logging.ERROR)
+
     # allow setting MQTT username and password via environment variables
     if not args.user and 'MQTT_USERNAME' in os.environ:
         args.user = os.environ['MQTT_USERNAME']
@@ -648,4 +712,7 @@ if __name__ == "__main__":
     if not args.password and 'MQTT_PASSWORD' in os.environ:
         args.password = os.environ['MQTT_PASSWORD']
 
+    if not args.user or not args.password:
+        logging.warning("User or password is not set. Check credentials if subscriptions do not return messages.")
+
     run()
