@@ -71,11 +71,13 @@ typedef struct r_cfg {
     uint64_t input_pos;
     uint32_t bytes_to_read;
     struct sdr_dev *dev;
-    int grab_mode;
+    int grab_mode; ///< Signal grabber mode: 0=off, 1=all, 2=unknown, 3=known
+    int raw_mode; ///< Raw pulses printing mode: 0=off, 1=all, 2=unknown, 3=known
     int verbosity; ///< 0=normal, 1=verbose, 2=verbose decoders, 3=debug decoders, 4=trace decoding.
     int verbose_bits;
     conversion_mode_t conversion_mode;
     int report_meta;
+    int report_noise;
     int report_protocol;
     time_mode_t report_time;
     int report_time_hires;
@@ -89,13 +91,11 @@ typedef struct r_cfg {
     int no_default_devices;
     struct r_device *devices;
     uint16_t num_r_devices;
-    char *output_key;
-    char *output_tag;
+    list_t data_tags;
     list_t output_handler;
     struct dm_state *demod;
     char const *sr_filename;
     int sr_execopen;
-    int old_model_keys;
     /* stats*/
     time_t frames_since; ///< stats start time
     unsigned frames_count; ///< stats counter for interval
