@@ -23,22 +23,20 @@ May be useful for other Cardin product too
 
 static int cardin_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
-    bitrow_t *bb = bitbuffer->bb;
+    uint8_t *b = bitbuffer->bb[0];
     unsigned char dip[10] = {'-','-','-','-','-','-','-','-','-', '\0'};
 
-
     char *rbutton[4] = { "11R", "10R", "01R", "00L?" };
-    data_t *data;
 
     if (bitbuffer->bits_per_row[0] != 24)
-      return DECODE_ABORT_LENGTH;
+        return DECODE_ABORT_LENGTH;
 
     // validate message as we can
-    if ((bb[0][2] & 48) == 0 && (
-                (bb[0][2] & 0x0f) == 3 ||
-                (bb[0][2] & 0x0f) == 9 ||
-                (bb[0][2] & 0x0f) == 12 ||
-                (bb[0][2] & 0x0f) == 6) ) {
+    if ((b[2] & 48) == 0 && (
+                (b[2] & 0x0f) == 3 ||
+                (b[2] & 0x0f) == 9 ||
+                (b[2] & 0x0f) == 12 ||
+                (b[2] & 0x0f) == 6) ) {
 
 /*
         fprintf(stderr, "------------------------------\n");
@@ -46,7 +44,7 @@ static int cardin_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         fprintf(stderr, "message        = ");
         for (i=0 ; i<3 ; i++) {
             for (k = 7; k >= 0; k--) {
-                if (bb[0][i] & 1 << k)
+                if (b[i] & 1 << k)
                     fprintf(stderr, "1");
                 else
                     fprintf(stderr, "0");
@@ -57,89 +55,90 @@ static int cardin_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 */
 
         // Dip 1
-        if (bb[0][0] & 8) {
+        if (b[0] & 8) {
             dip[0] = 'o';
-            if (bb[0][1] & 8)
+            if (b[1] & 8)
                 dip[0] = '+';
         }
         // Dip 2
-        if (bb[0][0] & 16) {
+        if (b[0] & 16) {
             dip[1] = 'o';
-            if (bb[0][1] & 16)
+            if (b[1] & 16)
                 dip[1] = '+';
         }
         // Dip 3
-        if (bb[0][0] & 32) {
+        if (b[0] & 32) {
             dip[2] = 'o';
-            if (bb[0][1] & 32)
+            if (b[1] & 32)
                 dip[2] = '+';
         }
         // Dip 4
-        if (bb[0][0] & 64) {
+        if (b[0] & 64) {
             dip[3] = 'o';
-            if (bb[0][1] & 64)
+            if (b[1] & 64)
                 dip[3] = '+';
         }
         // Dip 5
-        if (bb[0][0] & 128) {
+        if (b[0] & 128) {
             dip[4] = 'o';
-            if (bb[0][1] & 128)
+            if (b[1] & 128)
                 dip[4] = '+';
         }
         // Dip 6
-        if (bb[0][2] & 128) {
+        if (b[2] & 128) {
             dip[5] = 'o';
-            if (bb[0][2] & 64)
+            if (b[2] & 64)
                 dip[5] = '+';
         }
         // Dip 7
-        if (bb[0][0] & 1) {
+        if (b[0] & 1) {
             dip[6] = 'o';
-            if (bb[0][1] & 1)
+            if (b[1] & 1)
                 dip[6] = '+';
         }
         // Dip 8
-        if (bb[0][0] & 2) {
+        if (b[0] & 2) {
             dip[7] = 'o';
-            if (bb[0][1] & 2)
+            if (b[1] & 2)
                 dip[7] = '+';
         }
         // Dip 9
-        if (bb[0][0] & 4) {
+        if (b[0] & 4) {
             dip[8] = 'o';
-            if (bb[0][1] & 4)
+            if (b[1] & 4)
                 dip[8] = '+';
         }
 
-        data = data_make(
-            "model",      "",                       DATA_STRING, _X("Cardin-S466","Cardin S466"),
-            "dipswitch",  "dipswitch",              DATA_STRING, dip,
-            "rbutton",    "right button switches",  DATA_STRING, rbutton[((bb[0][2] & 15) / 3)-1],
-            NULL);
+        /* clang-format off */
+        data_t *data = data_make(
+                "model",      "",                       DATA_STRING, "Cardin-S466",
+                "dipswitch",  "dipswitch",              DATA_STRING, dip,
+                "rbutton",    "right button switches",  DATA_STRING, rbutton[((b[2] & 15) / 3)-1],
+                NULL);
+        /* clang-format on */
 
         decoder_output_data(decoder, data);
-
         return 1;
     }
     return DECODE_ABORT_EARLY;
 }
 
 static char *output_fields[] = {
-    "model",
-    "dipswitch",
-    "rbutton",
-    NULL,
+        "model",
+        "dipswitch",
+        "rbutton",
+        NULL,
 };
 
 r_device cardin = {
-    .name           = "Cardin S466-TX2",
-    .modulation     = OOK_PULSE_PWM,
-    .short_width    = 730,
-    .long_width     = 1400,
-    .sync_width     = 6150,
-    .gap_limit      = 1600,
-    .reset_limit    = 32000,
-    .decode_fn      = &cardin_callback,
-    .disabled       = 0,
-    .fields         = output_fields,
+        .name        = "Cardin S466-TX2",
+        .modulation  = OOK_PULSE_PWM,
+        .short_width = 730,
+        .long_width  = 1400,
+        .sync_width  = 6150,
+        .gap_limit   = 1600,
+        .reset_limit = 32000,
+        .decode_fn   = &cardin_callback,
+        .disabled    = 0,
+        .fields      = output_fields,
 };
