@@ -113,15 +113,12 @@ static int dsc_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     int s_closed, s_event, s_tamper, s_battery_low;
     int s_xactivity, s_xtamper1, s_xtamper2, s_exception;
 
-    if (decoder->verbose > 1) {
-        fprintf(stderr,"Possible DSC Contact: ");
-        bitbuffer_print(bitbuffer);
-    }
+    int result = 0;
 
     for (int row = 0; row < bitbuffer->num_rows; row++) {
         if (decoder->verbose > 1 && bitbuffer->bits_per_row[row] > 0 ) {
-            fprintf(stderr,"row %d bit count %d\n", row,
-                bitbuffer->bits_per_row[row]);
+            fprintf(stderr, "%s: row %d bit count %d\n", __func__,
+                    row, bitbuffer->bits_per_row[row]);
         }
 
         // Number of bits in the packet should be 48 but due to the
@@ -134,9 +131,10 @@ static int dsc_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         if (bitbuffer->bits_per_row[row] < 48 ||
             bitbuffer->bits_per_row[row] > 70) {  // should be 48 at most
             if (decoder->verbose > 1 && bitbuffer->bits_per_row[row] > 0) {
-                fprintf(stderr,"DSC row %d invalid bit count %d\n",
+                fprintf(stderr, "%s: row %d invalid bit count %d\n", __func__,
                         row, bitbuffer->bits_per_row[row]);
             }
+            result = DECODE_ABORT_EARLY;
             continue; // DECODE_ABORT_EARLY
         }
 
@@ -148,8 +146,9 @@ static int dsc_callback(r_device *decoder, bitbuffer_t *bitbuffer)
               (b[3] & 0x02) &&
               (b[4] & 0x01))) {
             if (decoder->verbose > 1) {
-                bitrow_printf(b, 40, "DSC Invalid start/sync bits ");
+                bitrow_printf(b, 40, "%s: Invalid start/sync bits ", __func__);
             }
+            result = DECODE_ABORT_EARLY;
             continue; // DECODE_ABORT_EARLY
         }
 
@@ -159,19 +158,26 @@ static int dsc_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         bytes[3] = ((b[3] & 0x01) << 7) | ((b[4] & 0xFE) >> 1);
         bytes[4] = ((b[5]));
 
+        // prevent false positive of: ff ff ff ff 00
+        if (bytes[0] == 0xff && bytes[1] == 0xff && bytes[2] == 0xff && bytes[3] == 0xff) {
+            result = DECODE_FAIL_SANITY;
+            continue; // DECODE_FAIL_SANITY
+        }
+
         if (decoder->verbose) {
-            bitrow_printf(bytes, 40, "DSC Contact Raw Data: ");
+            bitrow_printf(bytes, 40, "%s: Contact Raw Data: ", __func__);
         }
 
         status = bytes[0];
-        //subtype = bytes[1] >> 4;  // @todo needed for detecing keyfob
+        //subtype = bytes[1] >> 4;  // @todo needed for detecting keyfob
         esn = (bytes[1] << 16) | (bytes[2] << 8) | bytes[3];
         crc = bytes[4];
 
         if (crc8le(bytes, DSC_CT_MSGLEN, 0xf5, 0x3d) != 0) {
             if (decoder->verbose)
-                fprintf(stderr,"DSC Contact bad CRC: %06X, Status: %02X, CRC: %02X\n",
+                fprintf(stderr, "%s: Contact bad CRC: %06X, Status: %02X, CRC: %02X\n", __func__,
                         esn, status, crc);
+            result = DECODE_FAIL_MIC;
             continue; // DECODE_FAIL_MIC
         }
 
@@ -208,12 +214,12 @@ static int dsc_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
         /* clang-format off */
         data = data_make(
-                "model",        "",             DATA_STRING, _X("DSC-Security","DSC Contact"),
+                "model",        "",             DATA_STRING, "DSC-Security",
                 "id",           "",             DATA_INT,    esn,
                 "closed",       "",             DATA_INT,    s_closed, // @todo make bool
                 "event",        "",             DATA_INT,    s_event, // @todo make bool
                 "tamper",       "",             DATA_INT,    s_tamper, // @todo make bool
-                _X("battery_ok","battery_low"), "", DATA_INT, _X(!s_battery_low,s_battery_low),
+                "battery_ok",   "Battery",      DATA_INT,    !s_battery_low,
                 "xactivity",    "",             DATA_INT,    s_xactivity, // @todo make bool
 
                 // Note: the following may change or be removed
@@ -235,7 +241,8 @@ static int dsc_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         return 1;
     }
 
-    return 0;
+    // Only returns the latest result, but better than nothing.
+    return result;
 }
 
 static char *output_fields[] = {
