@@ -93,13 +93,13 @@ static int efergy_e2_classic_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     /* clang-format off */
     data = data_make(
-            "model",    "",                 DATA_STRING, _X("Efergy-e2CT","Efergy e2 CT"),
-            "id",       "Transmitter ID",   DATA_INT,    address,
-            "battery",  "Battery",          DATA_STRING, battery ? "OK" : "LOW",
-            "current",  "Current",          DATA_FORMAT, "%.2f A", DATA_DOUBLE, current_adc,
-            "interval", "Interval",         DATA_FORMAT, "%ds", DATA_INT, interval,
-            "learn",    "Learning",         DATA_STRING, learn ? "YES" : "NO",
-            "mic",      "Integrity",        DATA_STRING, "CHECKSUM",
+            "model",        "",                 DATA_STRING, "Efergy-e2CT",
+            "id",           "Transmitter ID",   DATA_INT,    address,
+            "battery_ok",   "Battery",          DATA_INT,    !!battery,
+            "current",      "Current",          DATA_FORMAT, "%.2f A", DATA_DOUBLE, current_adc,
+            "interval",     "Interval",         DATA_FORMAT, "%ds", DATA_INT, interval,
+            "learn",        "Learning",         DATA_STRING, learn ? "YES" : "NO",
+            "mic",          "Integrity",        DATA_STRING, "CHECKSUM",
             NULL);
     /* clang-format on */
 
@@ -110,7 +110,7 @@ static int efergy_e2_classic_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 static char *output_fields[] = {
         "model",
         "id",
-        "battery",
+        "battery_ok",
         "current",
         "interval",
         "learn",
