@@ -62,13 +62,18 @@ There is an extra, unidentified 7th byte in WH2A packages.
 Based on reverse engineering with gnu-radio and the nice article here:
 http://lucsmall.com/2012/04/29/weather-station-hacking-part-2/
 */
+#define MODEL_WH2 2
+#define MODEL_WH2A 3
+#define MODEL_WH5 5
+#define MODEL_RB 6
+#define MODEL_TP 7
 static int fineoffset_WH2_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
     bitrow_t *bb = bitbuffer->bb;
     uint8_t b[6] = {0};
     data_t *data;
 
-    char *model;
+    int model_num;
     int type;
     uint8_t id;
     int16_t temp;
@@ -78,24 +83,24 @@ static int fineoffset_WH2_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     if (bitbuffer->bits_per_row[0] == 48 &&
             bb[0][0] == 0xFF) { // WH2
         bitbuffer_extract_bytes(bitbuffer, 0, 8, b, 40);
-        model = _X("Fineoffset-WH2","Fine Offset Electronics, WH2 Temperature/Humidity sensor");
+        model_num = MODEL_WH2;
 
     } else if (bitbuffer->bits_per_row[0] == 55 &&
             bb[0][0] == 0xFE) { // WH2A
         bitbuffer_extract_bytes(bitbuffer, 0, 7, b, 48);
-        model = _X("Fineoffset-WH2A","Fine Offset WH2A sensor");
+        model_num = MODEL_WH2A;
 
     } else if (bitbuffer->bits_per_row[0] == 47 &&
             bb[0][0] == 0xFE) { // WH5
         bitbuffer_extract_bytes(bitbuffer, 0, 7, b, 40);
-        model = _X("Fineoffset-WH5","Fine Offset WH5 sensor");
+        model_num = MODEL_WH5;
         if (decoder->decode_ctx) // don't care for the actual value
-            model = "Rosenborg-66796";
+            model_num = MODEL_RB;
 
     } else if (bitbuffer->bits_per_row[0] == 49 &&
             bb[0][0] == 0xFF && (bb[0][1]&0x80) == 0x80) { // Telldus
         bitbuffer_extract_bytes(bitbuffer, 0, 9, b, 40);
-        model = _X("Fineoffset-TelldusProove","Telldus/Proove thermometer");
+        model_num = MODEL_TP;
 
     } else
         return DECODE_ABORT_LENGTH;
@@ -108,7 +113,7 @@ static int fineoffset_WH2_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     type = b[0] >> 4;
     if (type != 4) {
         if (decoder->verbose)
-            fprintf(stderr, "%s: Unknown type: %d\n", model, type);
+            fprintf(stderr, "%s: Unknown type: (%d) %d\n", __func__, model_num, type);
         return DECODE_FAIL_SANITY;
     }
 
@@ -132,31 +137,21 @@ static int fineoffset_WH2_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     // Nibble 8,9 contains humidity
     humidity = b[3];
 
-    // Thermo
-    if (b[3] == 0xFF) {
-        /* clang-format off */
-        data = data_make(
-                "model",            "",             DATA_STRING, model,
-                "id",               "ID",           DATA_INT, id,
-                "temperature_C",    "Temperature",  DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
-                "mic",              "Integrity",    DATA_STRING, "CRC",
-                NULL);
-        /* clang-format on */
-        decoder_output_data(decoder, data);
-    }
-    // Thermo/Hygro
-    else {
-        /* clang-format off */
-        data = data_make(
-                "model",            "",             DATA_STRING, model,
-                "id",               "ID",           DATA_INT, id,
-                "temperature_C",    "Temperature",  DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
-                "humidity",         "Humidity",     DATA_FORMAT, "%u %%", DATA_INT, humidity,
-                "mic",              "Integrity",    DATA_STRING, "CRC",
-                NULL);
-        /* clang-format on */
-        decoder_output_data(decoder, data);
-    }
+    /* clang-format off */
+    data = data_make(
+            "model",            "",             DATA_COND, model_num == MODEL_WH2,  DATA_STRING, "Fineoffset-WH2",
+            "model",            "",             DATA_COND, model_num == MODEL_WH2A, DATA_STRING, "Fineoffset-WH2A",
+            "model",            "",             DATA_COND, model_num == MODEL_WH5,  DATA_STRING, "Fineoffset-WH5",
+            "model",            "",             DATA_COND, model_num == MODEL_RB,   DATA_STRING, "Rosenborg-66796",
+            "model",            "",             DATA_COND, model_num == MODEL_TP,   DATA_STRING, "Fineoffset-TelldusProove",
+            "id",               "ID",           DATA_INT, id,
+            "temperature_C",    "Temperature",  DATA_FORMAT, "%.1f C", DATA_DOUBLE, temperature,
+            "humidity",         "Humidity",     DATA_COND, humidity != 0xff, DATA_FORMAT, "%u %%", DATA_INT, humidity,
+            "mic",              "Integrity",    DATA_STRING, "CRC",
+            NULL);
+    /* clang-format on */
+
+    decoder_output_data(decoder, data);
     return 1;
 }
 
@@ -307,15 +302,15 @@ static int fineoffset_WH24_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     /* clang-format off */
     data = data_make(
-            "model",            "",                 DATA_STRING, type == MODEL_WH24 ? _X("Fineoffset-WH24","Fine Offset WH24") : _X("Fineoffset-WH65B","Fine Offset WH65B"),
-            "id",               "ID",               DATA_INT, id,
-            "battery",          "Battery",          DATA_STRING, low_battery ? "LOW" : "OK",
+            "model",            "",                 DATA_STRING, type == MODEL_WH24 ? "Fineoffset-WH24" : "Fineoffset-WH65B",
+            "id",               "ID",               DATA_INT,    id,
+            "battery_ok",       "Battery",          DATA_INT,    !low_battery,
             "temperature_C",    "Temperature",      DATA_COND, temp_raw != 0x7ff, DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
             "humidity",         "Humidity",         DATA_COND, humidity != 0xff, DATA_FORMAT, "%u %%", DATA_INT, humidity,
             "wind_dir_deg",     "Wind direction",   DATA_COND, wind_dir != 0x1ff, DATA_INT, wind_dir,
-            _X("wind_avg_m_s","wind_speed_ms"),    "Wind speed",       DATA_COND, wind_speed_raw != 0x1ff, DATA_FORMAT, "%.1f m/s", DATA_DOUBLE, wind_speed_ms,
-            _X("wind_max_m_s","gust_speed_ms"),    "Gust speed",       DATA_COND, gust_speed_raw != 0xff, DATA_FORMAT, "%.1f m/s", DATA_DOUBLE, gust_speed_ms,
-            _X("rain_mm","rainfall_mm"),           "Rainfall",         DATA_FORMAT, "%.1f mm", DATA_DOUBLE, rainfall_mm,
+            "wind_avg_m_s",     "Wind speed",       DATA_COND, wind_speed_raw != 0x1ff, DATA_FORMAT, "%.1f m/s", DATA_DOUBLE, wind_speed_ms,
+            "wind_max_m_s",     "Gust speed",       DATA_COND, gust_speed_raw != 0xff, DATA_FORMAT, "%.1f m/s", DATA_DOUBLE, gust_speed_ms,
+            "rain_mm",          "Rainfall",         DATA_FORMAT, "%.1f mm", DATA_DOUBLE, rainfall_mm,
             "uv",               "UV",               DATA_COND, uv_raw != 0xffff, DATA_INT, uv_raw,
             "uvi",              "UVI",              DATA_COND, uv_raw != 0xffff, DATA_INT, uv_index,
             "light_lux",        "Light",            DATA_COND, light_raw != 0xffffff, DATA_FORMAT, "%.1f lux", DATA_DOUBLE, light_lux,
@@ -330,23 +325,75 @@ static int fineoffset_WH24_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 /**
 Fine Offset Electronics WH0290 Wireless Air Quality Monitor
 Also: Ambient Weather PM25
+Also: Misol PM25
+Also: EcoWitt WH0290, EcoWitt WH41
+
+The sensor sends a data burst every 10 minutes.  The bits are PCM
+modulated with Frequency Shift Keying.
+
+Ecowitt advertises this device as a PM2.5 sensor.  It contains a
+Honeywell PM2.5 sensor:
+
+  https://sensing.honeywell.com/honeywell-sensing-particulate-hpm-series-datasheet-32322550.pdf
+
+However, the Honeywell datasheet says that it also has a PM10 output
+which is "calculated from" the PM2.5 reading.  While there is an
+accuracy spec for PM2.5, there is no specification of an kind from
+PM10.  The datasheet does not explain the calculation, and does not
+give references to papers in the scientific literature.
+
+Note that PM2.5 is the mass of particles <= 2.5 microns in 1 m^3 of
+air, and PM10 is the mass of particles <= 10 microns.  Therefore the
+difference in those measurements is the mass of particles > 2.5
+microns and <= 10 microns, sometimes called PM2.5-10.  By definition
+these particles are not included in the PM2.5 measurement, so
+"calculating" doesn't make sense.  Rather, this appears an assumption
+about correlation, meaning how much mass of larger particles is likely
+to be present based on the mass of the smaller particles.
+
+The serial stream from the sensor has fields for PM2.5 and PM10 and
+these fields have been verified to appear in the transmitted signal by
+cross-comparing the internal serial lines and data received via
+rtl_433.
+
+The Ecowitt displays show only PM2.5, and Ecowitt confirmed that the
+second field is the PM10 output of the sensor but said the value is
+not accurate so they have not adopted it.
+
+By observation of an Ecowitt WH41, the formula is pm10 = pm2.5 +
+increment(pm2.5), where the increment is by ranges from the following
+table (with gaps when no samples have been observed).  It is left as
+future work to compare with an actual PM10 sensor.
+
+0 to 24     | 1
+25 to 106   | 2
+109 to 185  | 3
+190 to 222  | 4
+311         | 5
+390         | 6
 
-The sensor sends a package each ~10m. The bits are PCM modulated with Frequency Shift Keying.
 
 Data layout:
+             41 c7 41 ae 01 c2 f9 b3 00000, Ecowitt 41
     aa 2d d4 42 cc 41 9a 41 ae c1 99 9
              FF DD ?P PP ?A AA CC BB
 
 - F: 8 bit Family Code?
-- D: 8 bit device id?
-- ?: 2 bits ?
+- D: 8 bit device id (corresponds to sticker on device in hex)
+- ?: 1 bit?
+- b: 1 bit MSB of battery bars out of 5
 - P: 14 bit PM2.5 reading in ug/m3
-- ?: 2 bits ?
+- b: 2 bits LSBs of battery bars out of 5
 - A: 14 bit PM10.0 reading in ug/m3
-- ?: 8 bits ?
 - C: 8 bit CRC checksum of the previous 6 bytes
 - B: 8 bit Bitsum (sum without carry, XOR) of the previous 7 bytes
 
+BitBench Examples
+{129} 55 55 55 55 55 51 6e a2 0c ba 02 d0 03 25 13 c0 00 [pm2=9 pm10=10 id=151 0x97 battery 4/5bars]
+{128} 55 55 55 55 55 51 6e a2 0c ba 03 70 03 c3 43 30 [pm2=11 pm10=12 id=151 0x97 battery 4/5bars]
+{129} 55 55 55 55 55 51 6e a2 0c b8 01 46 01 94 9c 2c 00 [pm2=4 pm10=5 id=151 0x97 3/5 bars]
+Preamble: aa2dd4
+FAM:8d ID: 8h 1b Bat_MSB:1d PMTWO:14d Bat_LSB:2d PMTEN:14d CRC:8h BITSIM:8h bbbbb
 */
 static int fineoffset_WH0290_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
@@ -358,7 +405,7 @@ static int fineoffset_WH0290_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     bit_offset = bitbuffer_search(bitbuffer, 0, 0, preamble, sizeof(preamble) * 8) + sizeof(preamble) * 8;
     if (bit_offset + sizeof(b) * 8 > bitbuffer->bits_per_row[0]) {  // Did not find a big enough package
         if (decoder->verbose)
-            bitbuffer_printf(bitbuffer, "Fineoffset_WH0290: short package. Row length: %u. Header index: %u\n", bitbuffer->bits_per_row[0], bit_offset);
+            bitbuffer_printf(bitbuffer, "%s: short package. Row length: %u. Header index: %u\n", __func__, bitbuffer->bits_per_row[0], bit_offset);
         return DECODE_ABORT_LENGTH;
     }
     bitbuffer_extract_bytes(bitbuffer, 0, bit_offset, b, sizeof(b) * 8);
@@ -371,23 +418,29 @@ static int fineoffset_WH0290_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     }
     if (crc != b[6] || checksum != b[7]) {
         if (decoder->verbose) {
-            fprintf(stderr, "Fineoffset_WH0280: Checksum error: %02x %02x\n", crc, checksum);
+            fprintf(stderr, "%s: Checksum error: %02x %02x\n", __func__, crc, checksum);
         }
         return DECODE_FAIL_MIC;
     }
 
     // Decode data
+    uint8_t family    = b[0];
     uint8_t id        = b[1];
+    uint8_t unknown1  = (b[2] & 0x80) ? 1 : 0;
     int pm25          = (b[2] & 0x3f) << 8 | b[3];
     int pm100         = (b[4] & 0x3f) << 8 | b[5];
-
+    int battery_bars  = (b[2] & 0x40) >> 4 | (b[4] & 0xC0) >> 6; //out of 5
+    float battery_ok  = battery_bars * 0.2f; //convert out of 5 bars to 0 (0 bars) to 1 (5 bars)
 
     /* clang-format off */
     data = data_make(
-            "model",            "",             DATA_STRING, _X("Fineoffset-WH0290","Fine Offset Electronics, WH0290"),
+            "model",            "",             DATA_STRING, "Fineoffset-WH0290",
             "id",               "ID",           DATA_INT,    id,
+            "battery_ok",       "Battery Level",  DATA_FORMAT, "%.1f", DATA_DOUBLE, battery_ok,
             "pm2_5_ug_m3",      "2.5um Fine Particulate Matter",  DATA_FORMAT, "%i ug/m3", DATA_INT, pm25/10,
-            "pm10_0_ug_m3",     "10um Coarse Particulate Matter",  DATA_FORMAT, "%i ug/m3", DATA_INT, pm100/10,
+            "estimated_pm10_0_ug_m3",     "Estimate of 10um Coarse Particulate Matter",  DATA_FORMAT, "%i ug/m3", DATA_INT, pm100/10,
+            "family",           "FAMILY",       DATA_INT,    family,
+            "unknown1",         "UNKNOWN1",     DATA_INT,    unknown1,
             "mic",              "Integrity",    DATA_STRING, "CRC",
             NULL);
     /* clang-format on */
@@ -449,7 +502,7 @@ static int fineoffset_WH25_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     bit_offset = bitbuffer_search(bitbuffer, 0, 100, preamble, sizeof(preamble) * 8) + sizeof(preamble) * 8;
     if (bit_offset + sizeof(b) * 8 > bitbuffer->bits_per_row[0]) {  // Did not find a big enough package
         if (decoder->verbose)
-            bitbuffer_printf(bitbuffer, "Fineoffset_WH25: short package. Header index: %u\n", bit_offset);
+            bitbuffer_printf(bitbuffer, "%s: short package. Header index: %u\n", __func__, bit_offset);
         return DECODE_ABORT_LENGTH;
     }
     bitbuffer_extract_bytes(bitbuffer, 0, bit_offset, b, sizeof(b) * 8);
@@ -462,7 +515,10 @@ static int fineoffset_WH25_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     }
     else if (msg_type != 0xe0) {
         if (decoder->verbose)
-            fprintf(stderr, "Fineoffset_WH25: Msg type unknown: %2x\n", b[0]);
+            fprintf(stderr, "%s: Msg type unknown: %2x\n", __func__, b[0]);
+        if (b[0] == 0x41) {
+            return fineoffset_WH0290_callback(decoder, bitbuffer); // abort and try WH0290
+        }
         return DECODE_ABORT_EARLY;
     }
 
@@ -470,7 +526,7 @@ static int fineoffset_WH25_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     int sum = (add_bytes(b, 6) & 0xff) - b[6];
     if (sum) {
         if (decoder->verbose)
-            bitrow_printf(b, sizeof (b) * 8, "Fineoffset_WH25: Checksum error: ");
+            bitrow_printf(b, sizeof (b) * 8, "%s: Checksum error: ", __func__);
         return DECODE_FAIL_MIC;
     }
 
@@ -479,7 +535,7 @@ static int fineoffset_WH25_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     bitsum = ((bitsum & 0x0f) << 4) | (bitsum >> 4); // Swap nibbles
     if (type == 25 && bitsum != b[7]) { // only check for WH25
         if (decoder->verbose)
-            bitrow_printf(b, sizeof (b) * 8, "Fineoffset_WH25: Bitsum error: ");
+            bitrow_printf(b, sizeof (b) * 8, "%s: Bitsum error: ", __func__);
         return DECODE_FAIL_MIC;
     }
 
@@ -497,9 +553,9 @@ static int fineoffset_WH25_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     data = data_make(
             "model",            "",             DATA_COND, type == 31, DATA_STRING, "Fineoffset-WH32",
             "model",            "",             DATA_COND, type == 32, DATA_STRING, "Fineoffset-WH32B",
-            "model",            "",             DATA_COND, type == 25, DATA_STRING, _X("Fineoffset-WH25","Fine Offset Electronics, WH25"),
+            "model",            "",             DATA_COND, type == 25, DATA_STRING, "Fineoffset-WH25",
             "id",               "ID",           DATA_INT,    id,
-            "battery",          "Battery",      DATA_STRING, low_battery ? "LOW" : "OK",
+            "battery_ok",       "Battery",      DATA_INT,    !low_battery,
             "temperature_C",    "Temperature",  DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
             "humidity",         "Humidity",     DATA_FORMAT, "%u %%", DATA_INT, humidity,
             "pressure_hPa",     "Pressure",     DATA_COND,   pressure_raw != 0xffff, DATA_FORMAT, "%.01f hPa", DATA_DOUBLE, pressure,
@@ -514,6 +570,8 @@ static int fineoffset_WH25_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 /*
 Fine Offset WH51, ECOWITT WH51, MISOL/1 Soil Moisture Sensor
 
+Also: SwitchDoc Labs SM23 Soil Moisture Sensor.
+
 Test decoding with: rtl_433 -f 433920000  -X "n=soil_sensor,m=FSK_PCM,s=58,l=58,t=5,r=5000,g=4000,preamble=aa2dd4"
 
 Data format:
@@ -563,7 +621,7 @@ static int fineoffset_WH51_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     bit_offset = bitbuffer_search(bitbuffer, 0, 0, preamble, sizeof(preamble) * 8) + sizeof(preamble) * 8;
     if (bit_offset + sizeof(b) * 8 > bitbuffer->bits_per_row[0]) {  // Did not find a big enough package
         if (decoder->verbose)
-            bitbuffer_printf(bitbuffer, "Fineoffset_WH51: short package. Header index: %u\n", bit_offset);
+            bitbuffer_printf(bitbuffer, "%s: short package. Header index: %u\n", __func__, bit_offset);
         return DECODE_ABORT_LENGTH;
     }
     bitbuffer_extract_bytes(bitbuffer, 0, bit_offset, b, sizeof(b) * 8);
@@ -571,21 +629,21 @@ static int fineoffset_WH51_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     // Verify family code
     if (b[0] != 0x51) {
         if (decoder->verbose)
-            fprintf(stderr, "Fineoffset_WH51: Msg family unknown: %2x\n", b[0]);
+            fprintf(stderr, "%s: Msg family unknown: %2x\n", __func__, b[0]);
         return DECODE_ABORT_EARLY;
     }
 
     // Verify checksum
     if ((add_bytes(b, 13) & 0xff) != b[13]) {
         if (decoder->verbose)
-            bitrow_printf(b, sizeof (b) * 8, "Fineoffset_WH51: Checksum error: ");
+            bitrow_printf(b, sizeof (b) * 8, "%s: Checksum error: ", __func__);
         return DECODE_FAIL_MIC;
     }
 
     // Verify crc
     if (crc8(b, 12, 0x31, 0) != b[12]) {
         if (decoder->verbose)
-            bitrow_printf(b, sizeof (b) * 8, "Fineoffset_WH51: Bitsum error: ");
+            bitrow_printf(b, sizeof (b) * 8, "%s: Bitsum error: ", __func__);
         return DECODE_FAIL_MIC;
     }
 
@@ -672,10 +730,10 @@ static int alecto_ws1200v1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     /* clang-format off */
     data = data_make(
             "model",            "",             DATA_STRING, "Alecto-WS1200v1",
-            "id",               "ID",           DATA_INT, id,
-            "battery",          "Battery",      DATA_STRING, battery_low ? "LOW" : "OK",
+            "id",               "ID",           DATA_INT,    id,
+            "battery_ok",       "Battery",      DATA_INT,    !battery_low,
             "temperature_C",    "Temperature",  DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
-            "rain",             "Rain",         DATA_FORMAT, "%.01f mm", DATA_DOUBLE, rainfall,
+            "rain_mm",          "Rain",         DATA_FORMAT, "%.01f mm", DATA_DOUBLE, rainfall,
             "mic",              "Integrity",    DATA_STRING, "CRC",
             NULL);
     /* clang-format on */
@@ -758,8 +816,8 @@ static int alecto_ws1200v2_dcf_callback(r_device *decoder, bitbuffer_t *bitbuffe
     /* clang-format off */
     data = data_make(
             "model",            "",             DATA_STRING, "Alecto-WS1200v2",
-            "id",               "ID",           DATA_INT, id,
-            "battery",          "Battery",      DATA_STRING, battery_low ? "LOW" : "OK",
+            "id",               "ID",           DATA_INT,    id,
+            "battery_ok",       "Battery",      DATA_INT,    !battery_low,
             "radio_clock",      "Radio Clock",  DATA_STRING, clock_str,
             "mic",              "Integrity",    DATA_STRING, "CRC",
             NULL);
@@ -834,10 +892,10 @@ static int alecto_ws1200v2_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     /* clang-format off */
     data = data_make(
             "model",            "",             DATA_STRING, "Alecto-WS1200v2",
-            "id",               "ID",           DATA_INT, id,
-            "battery",          "Battery",      DATA_STRING, battery_low ? "LOW" : "OK",
+            "id",               "ID",           DATA_INT,    id,
+            "battery_ok",       "Battery",      DATA_INT,    !battery_low,
             "temperature_C",    "Temperature",  DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
-            "rain",             "Rain",         DATA_FORMAT, "%.01f mm", DATA_DOUBLE, rainfall,
+            "rain_mm",          "Rain",         DATA_FORMAT, "%.01f mm", DATA_DOUBLE, rainfall,
             "mic",              "Integrity",    DATA_STRING, "CRC",
             NULL);
     /* clang-format on */
@@ -907,11 +965,11 @@ static int fineoffset_WH0530_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     /* clang-format off */
     data = data_make(
-            "model",            "",             DATA_STRING, _X("Fineoffset-WH0530","Fine Offset Electronics, WH0530 Temperature/Rain sensor"),
-            "id",               "ID",           DATA_INT, id,
-            "battery",          "Battery",      DATA_STRING, battery_low ? "LOW" : "OK",
+            "model",            "",             DATA_STRING, "Fineoffset-WH0530",
+            "id",               "ID",           DATA_INT,    id,
+            "battery_ok",       "Battery",      DATA_INT,    !battery_low,
             "temperature_C",    "Temperature",  DATA_FORMAT, "%.01f C", DATA_DOUBLE, temperature,
-            _X("rain_mm","rain"),             "Rain",         DATA_FORMAT, "%.01f mm", DATA_DOUBLE, rainfall,
+            "rain_mm",          "Rain",         DATA_FORMAT, "%.01f mm", DATA_DOUBLE, rainfall,
             "mic",              "Integrity",    DATA_STRING, "CRC",
             NULL);
     /* clang-format on */
@@ -932,24 +990,21 @@ static char *output_fields[] = {
 static char *output_fields_WH25[] = {
     "model",
     "id",
-    "battery",
+    "battery_ok",
     "temperature_C",
     "humidity",
     "pressure_hPa",
     // WH24
     "wind_dir_deg",
-    "wind_speed_ms", // TODO: delete this
-    "gust_speed_ms", // TODO: delete this
     "wind_avg_m_s",
     "wind_max_m_s",
-    "rainfall_mm", //TODO: remove this
     "rain_mm",
     "uv",
     "uvi",
     "light_lux",
     //WH0290
     "pm2_5_ug_m3",
-    "pm10_0_ug_m3",
+    "estimated_pm10_0_ug_m3",
     "mic",
     NULL,
 };
@@ -969,9 +1024,8 @@ static char *output_fields_WH51[] = {
 static char *output_fields_WH0530[] = {
     "model",
     "id",
-    "battery",
+    "battery_ok",
     "temperature_C",
-    "rain", //TODO: remove this
     "rain_mm",
     "radio_clock",
     "mic",
@@ -1003,7 +1057,7 @@ r_device fineoffset_WH25 = {
 };
 
 r_device fineoffset_WH51 = {
-    .name           = "Fine Offset Electronics/ECOWITT WH51 Soil Moisture Sensor",
+    .name           = "Fine Offset Electronics/ECOWITT WH51, SwitchDoc Labs SM23 Soil Moisture Sensor",
     .modulation     = FSK_PULSE_PCM,
     .short_width    = 58, // Bit width = 58µs (measured across 580 samples / 40 bits / 250 kHz )
     .long_width     = 58, // NRZ encoding (bit width = pulse width)
