@@ -20,7 +20,7 @@ static inline int bit(const uint8_t *bytes, unsigned bit)
 }
 
 /// extract all mask bits skipping unmasked bits of a number up to 32/64 bits
-unsigned long compact_number(uint8_t *data, unsigned bit_offset, unsigned long mask)
+static unsigned long compact_number(uint8_t *data, unsigned bit_offset, unsigned long mask)
 {
     // clz (fls) is not worth the trouble
     int top_bit = 0;
@@ -38,7 +38,7 @@ unsigned long compact_number(uint8_t *data, unsigned bit_offset, unsigned long m
 }
 
 /// extract a number up to 32/64 bits from given offset with given bit length
-unsigned long extract_number(uint8_t *data, unsigned bit_offset, unsigned bit_count)
+static unsigned long extract_number(uint8_t *data, unsigned bit_offset, unsigned bit_count)
 {
     unsigned pos = bit_offset / 8;            // the first byte we need
     unsigned shl = bit_offset - pos * 8;      // shift left we need to align
@@ -89,9 +89,9 @@ struct flex_params {
     unsigned unique;
     unsigned count_only;
     unsigned match_len;
-    bitrow_t match_bits;
+    uint8_t match_bits[128];
     unsigned preamble_len;
-    bitrow_t preamble_bits;
+    uint8_t preamble_bits[128];
     struct flex_get getter[GETTER_SLOTS];
     unsigned decode_uart;
 };
@@ -142,13 +142,12 @@ static void render_getters(data_t *data, uint8_t *bits, struct flex_params *para
 
 static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
-    int i, j;
+    int i;
     int match_count = 0;
     data_t *data;
     data_t *row_data[BITBUF_ROWS];
     char *row_codes[BITBUF_ROWS];
-    char row_bytes[BITBUF_COLS * 2 + 1];
-    bitrow_t tmp;
+    char row_bytes[BITBUF_ROWS * BITBUF_COLS * 2 + 1]; // TODO: this is a lot of stack
 
     struct flex_params *params = decoder->decode_ctx;
 
@@ -179,9 +178,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     if (params->reflect) {
         // TODO: refactor to utils
         for (i = 0; i < bitbuffer->num_rows; ++i) {
-            for (j = 0; j < (bitbuffer->bits_per_row[i] + 7) / 8; ++j) {
-                bitbuffer->bb[i][j] = reverse8(bitbuffer->bb[i][j]);
-            }
+            reflect_bytes(bitbuffer->bb[i], (bitbuffer->bits_per_row[i] + 7) / 8);
         }
     }
 
@@ -213,8 +210,9 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
                 pos += params->preamble_len;
                 // TODO: refactor to bitbuffer_shift_row()
                 unsigned len = bitbuffer->bits_per_row[i] - pos;
-                bitbuffer_extract_bytes(bitbuffer, i, pos, tmp, len);
-                memcpy(bitbuffer->bb[i], tmp, (len + 7) / 8);
+                bitbuffer_t tmp = {0};
+                bitbuffer_extract_bytes(bitbuffer, i, pos, tmp.bb[0], len);
+                memcpy(bitbuffer->bb[i], tmp.bb[0], (len + 7) / 8);
                 bitbuffer->bits_per_row[i] = len;
             }
         }
@@ -224,8 +222,11 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     if (params->decode_uart) {
         for (i = 0; i < bitbuffer->num_rows; i++) {
-            int len = extract_bytes_uart(bitbuffer->bb[i], 0, bitbuffer->bits_per_row[i], tmp);
-            memcpy(bitbuffer->bb[i], tmp, len);
+            // TODO: refactor to bitbuffer_decode_uart_row()
+            unsigned len = bitbuffer->bits_per_row[i];
+            bitbuffer_t tmp = {0};
+            len = extract_bytes_uart(bitbuffer->bb[i], 0, len, tmp.bb[0]);
+            memcpy(bitbuffer->bb[i], tmp.bb[0], len);
             bitbuffer->bits_per_row[i] = len * 8;
         }
     }
@@ -241,7 +242,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
         /* clang-format off */
         data = data_make(
-                "model", "", DATA_STRING, params->name,
+                "model", "", DATA_STRING, params->name, // "User-defined model"
                 "count", "", DATA_INT, match_count,
                 "num_rows", "", DATA_INT, bitbuffer->num_rows,
                 "len", "", DATA_INT, bitbuffer->bits_per_row[r],
@@ -259,7 +260,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     if (params->count_only) {
         /* clang-format off */
         data = data_make(
-                "model", "", DATA_STRING, params->name,
+                "model", "", DATA_STRING, params->name, // "User-defined model"
                 "count", "", DATA_INT, match_count,
                 NULL);
         /* clang-format on */
@@ -290,7 +291,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     }
     /* clang-format off */
     data = data_make(
-            "model", "", DATA_STRING, params->name,
+            "model", "", DATA_STRING, params->name, // "User-defined model"
             "count", "", DATA_INT, match_count,
             "num_rows", "", DATA_INT, bitbuffer->num_rows,
             "rows", "", DATA_ARRAY, data_array(bitbuffer->num_rows, DATA_DATA, row_data),
@@ -337,6 +338,7 @@ static void help()
             "\treset=<reset> (or: r=<reset>)\n"
             "\tgap=<gap> (or: g=<gap>)\n"
             "\ttolerance=<tolerance> (or: t=<tolerance>)\n"
+            "\tpriority=<n> : run decoder only as fallback\n"
             "where:\n"
             "<name> can be any descriptive name tag you need in the output\n"
             "<modulation> is one of:\n"
@@ -410,19 +412,25 @@ static unsigned parse_modulation(char const *str)
     return 0;
 }
 
-static unsigned parse_bits(const char *code, bitrow_t bitrow)
+// used for match, preamble, getter, limited to 1024 bits (128 byte).
+static unsigned parse_bits(const char *code, uint8_t *bitrow)
 {
     bitbuffer_t bits = {0};
     bitbuffer_parse(&bits, code);
     if (bits.num_rows != 1) {
-        fprintf(stderr, "Bad flex spec, \"match\" needs exactly one bit row (%d found)!\n", bits.num_rows);
+        fprintf(stderr, "Bad flex spec, \"match\", \"preamble\", and getter mask need exactly one bit row (%d found)!\n", bits.num_rows);
         usage();
     }
-    memcpy(bitrow, bits.bb[0], sizeof(bitrow_t));
-    return bits.bits_per_row[0];
+    unsigned len = bits.bits_per_row[0];
+    if (len > 1024) {
+        fprintf(stderr, "Bad flex spec, \"match\", \"preamble\", and getter mask mayb have up to 1024 bits (%d found)!\n", len);
+        usage();
+    }
+    memcpy(bitrow, bits.bb[0], (len + 7) / 8);
+    return len;
 }
 
-const char *parse_map(const char *arg, struct flex_get *getter)
+static const char *parse_map(const char *arg, struct flex_get *getter)
 {
     const char *c = arg;
     int i = 0;
@@ -466,7 +474,7 @@ const char *parse_map(const char *arg, struct flex_get *getter)
 
 static void parse_getter(const char *arg, struct flex_get *getter)
 {
-    bitrow_t bitrow;
+    uint8_t bitrow[128];
     while (arg && *arg) {
         if (*arg == '[') {
             arg = parse_map(arg, getter);
@@ -504,6 +512,9 @@ static void parse_getter(const char *arg, struct flex_get *getter)
     */
 }
 
+// NOTE: this is declared in rtl_433.c also.
+r_device *flex_create_device(char *spec);
+
 r_device *flex_create_device(char *spec)
 {
     if (!spec || !*spec || *spec == '?' || !strncasecmp(spec, "help", strlen(spec))) {
@@ -563,6 +574,8 @@ r_device *flex_create_device(char *spec)
             dev->reset_limit = atoi(val);
         else if (!strcasecmp(key, "t") || !strcasecmp(key, "tolerance"))
             dev->tolerance = atoi(val);
+        else if (!strcasecmp(key, "prio") || !strcasecmp(key, "priority"))
+            dev->priority = atoi(val);
 
         else if (!strcasecmp(key, "bits>"))
             params->min_bits = val ? atoi(val) : 0;
