@@ -3,9 +3,9 @@
 
     Copyright (C) 2019 Olaf Glage
 
-    This program is free software: you can redistribute it and/or modify
+    This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
+    the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 */
 /**
@@ -75,10 +75,6 @@ static int gt_tmbbq05_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     uint8_t b[4],p[4];
     data_t *data;
 
-    if (decoder->verbose > 1) {
-        bitbuffer_printf(bitbuffer, "%s: Possible Quigg BBQ: ", __func__);
-    }
-
     // 33 bit, repeated multiple times (technically it is repeated 8 times, look for 5 identical versions)
     int r = bitbuffer_find_repeated_row(bitbuffer, 5, 33);
 
@@ -112,11 +108,13 @@ static int gt_tmbbq05_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 
     int sum = add_nibbles(b, 3) + (b[3] >> 4);
     if ((sum & 0xf) != (b[3] & 0xf)) {
-        bitrow_printf(b, 32, "%s: Bad checksum (%x) ", __func__, sum);
+        if (decoder->verbose > 1) {
+            bitrow_printf(b, 32, "%s: Bad checksum (%x) ", __func__, sum);
+        }
         return DECODE_FAIL_MIC;
     }
 
-    // temperature: concat the upper bits to the lower bits and substract the fixed offset 90
+    // temperature: concat the upper bits to the lower bits and subtract the fixed offset 90
     int tempf = (((b[3] & 0xc0) << 2) | b[1]) - 90;
 
     // device id: concat the two bytes
