@@ -3,9 +3,9 @@
 
     Copyright (C) 2019 Christian W. Zuckschwerdt <zany@triq.net>
 
-    This program is free software: you can redistribute it and/or modify
+    This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
+    the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 */
 /** @fn int gt_wt_03_decode(r_device *decoder, bitbuffer_t *bitbuffer)
@@ -127,8 +127,8 @@ static int gt_wt_03_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     int battery_low    = (b[2] >> 7 & 1); // 1 bits
     int button_pressed = (b[2] >> 6 & 1); // 1 bits
     int channel        = (b[2] >> 4 & 3); // 2 bits
-    int temp_raw       = (int16_t)(((b[2] & 0x0f) << 12) | b[3] << 4) >> 4; // uses sign extend
-    float temp_c       = temp_raw * 0.1F;
+    int temp_raw       = (int16_t)(((b[2] & 0x0f) << 12) | (b[3] << 4)); // uses sign extend
+    float temp_c       = (temp_raw >> 4) * 0.1F;
 
     /* clang-format off */
     data = data_make(
