@@ -33,7 +33,7 @@ The data is sent using CPFSK modulation. It requires PD_MIN_PULSE_SAMPLES in
 pulse_detect.h to be lowered to 5 to be able to demodulate at 250kS/s. The
 preamble is optimally 4 bytes of 0XAA. Then the sync word 0xD201. Here only
 the last 2 bytes of the 0xAA preamble is checked, as the first ones seems
-to be corrupted quite often. There are plenty of integrety checks made on
+to be corrupted quite often. There are plenty of integrity checks made on
 the demodulated package which makes this compromise OK.
 
 Packet structure according to: https://github.com/strigeus/sparsnas_decoder
@@ -54,8 +54,8 @@ Packet structure according to: https://github.com/strigeus/sparsnas_decoder
 Example packet: 0x11a15f070ea2dfefe6d5fdd20547e6340ae7be61
 
 
-The packet's integrety can be checked with the 16b CRC at the end of the packet.
-There are also several other ways to check the integrety of the package.
+The packet's integrity can be checked with the 16b CRC at the end of the packet.
+There are also several other ways to check the integrity of the package.
  - (preamble)
  - CRC
  - The decrypted sensor ID
@@ -78,7 +78,6 @@ detail. Many thanks to kodarn!
 #define IKEA_SPARSNAS_MESSAGE_BITLEN_MAX 260 // Just for early sanity checks
 
 #define IKEA_SPARSNAS_PREAMBLE_BITLEN 32
-static const uint8_t preamble_pattern[4] = {0xAA, 0xAA, 0xD2, 0x01};
 
 #define IKEA_SPARSNAS_CRC_INIT 0xffff
 #define IKEA_SPARSNAS_CRC_POLY 0x8005
@@ -135,6 +134,7 @@ static uint32_t ikea_sparsnas_brute_force_encryption(uint8_t buffer[18])
 
 static int ikea_sparsnas_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
+    uint8_t const preamble_pattern[4] = {0xAA, 0xAA, 0xD2, 0x01};
 
     if ((bitbuffer->bits_per_row[0] < IKEA_SPARSNAS_MESSAGE_BITLEN) || (bitbuffer->bits_per_row[0] > IKEA_SPARSNAS_MESSAGE_BITLEN_MAX)) {
         if (decoder->verbose > 1) {
@@ -145,7 +145,7 @@ static int ikea_sparsnas_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     }
 
     // Look for preamble
-    uint16_t bitpos = bitbuffer_search(bitbuffer, 0, 0, (const uint8_t *)&preamble_pattern, IKEA_SPARSNAS_PREAMBLE_BITLEN);
+    uint16_t bitpos = bitbuffer_search(bitbuffer, 0, 0, preamble_pattern, IKEA_SPARSNAS_PREAMBLE_BITLEN);
 
     if ((bitbuffer->bits_per_row[0] == bitpos) || (bitpos + IKEA_SPARSNAS_MESSAGE_BITLEN > bitbuffer->bits_per_row[0])) {
         if (decoder->verbose > 1) {
@@ -265,19 +265,18 @@ static int ikea_sparsnas_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     //}
     float cumulative_kWh = ((float)pulses) / ((float)ikea_sparsnas_pulses_per_kwh);
 
-    data_t *data;
     /* clang-format off */
-    data = data_make(
+    data_t *data = data_make(
             "model",         "Model",               DATA_STRING, "Ikea-Sparsnas",
-            "id",            "Sensor ID",           DATA_INT, rcv_sensor_id,
-            "sequence",      "Sequence Number",     DATA_INT, sequence_number,
-            "battery",       "Battery",             DATA_FORMAT, "%d%%", DATA_INT, battery,
-            "pulses_per_kWh", "Pulses per kWh",     DATA_INT, ikea_sparsnas_pulses_per_kwh,
+            "id",            "Sensor ID",           DATA_INT,    rcv_sensor_id,
+            "sequence",      "Sequence Number",     DATA_INT,    sequence_number,
+            "battery_ok",    "Battery level",       DATA_INT,    battery * 0.01f, // 0-100
+            "pulses_per_kWh", "Pulses per kWh",     DATA_INT,    ikea_sparsnas_pulses_per_kwh,
             "cumulative_kWh", "Cumulative kWh",     DATA_FORMAT, "%7.3fkWh", DATA_DOUBLE,  cumulative_kWh,
             "effect",        "Effect",              DATA_FORMAT, "%dW", DATA_INT,  effect,
-            "pulses",        "Pulses",              DATA_INT,  pulses,
-            "mode",          "Mode",                DATA_INT, mode,
-            "mic",           "Integrity",           DATA_STRING,    "CRC",
+            "pulses",        "Pulses",              DATA_INT,    pulses,
+            "mode",          "Mode",                DATA_INT,    mode,
+            "mic",           "Integrity",           DATA_STRING, "CRC",
             NULL);
     /* clang-format on */
 
@@ -289,7 +288,7 @@ static char *output_fields[] = {
     "model",
     "id",
     "sequence",
-    "battery",
+    "battery_ok",
     "pulses_per_kwh",
     "cumulative_kWh",
     "effect",
