@@ -125,14 +125,16 @@ static int lacrossetx_detect(r_device *decoder, uint8_t *pRow, uint8_t *msg_nybb
 // Temperature and Humidity are sent in different messages bursts.
 static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
-    bitrow_t *bb = bitbuffer->bb;
     int events = 0;
     uint8_t msg_nybbles[LACROSSE_NYBBLE_CNT];
     data_t *data;
 
+    int result = 0;
+
     for (int row = 0; row < bitbuffer->num_rows; ++row) {
         // break out the message nybbles into separate bytes
-        if (lacrossetx_detect(decoder, bb[row], msg_nybbles, bitbuffer->bits_per_row[row]) <= 0) {
+        if (lacrossetx_detect(decoder, bitbuffer->bb[row], msg_nybbles, bitbuffer->bits_per_row[row]) <= 0) {
+            result = DECODE_ABORT_EARLY;
             continue; // DECODE_ABORT_EARLY
         }
 
@@ -152,6 +154,7 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                         "LaCrosse TX Sensor %02x, type: %d: message value mismatch int(%3.1f) != %d?\n",
                         sensor_id, msg_type, msg_value, msg_value_int);
             }
+            result = DECODE_FAIL_SANITY;
             continue; // DECODE_FAIL_SANITY
         }
 
@@ -159,7 +162,7 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             float temp_c = msg_value - 50.0;
             /* clang-format off */
             data = data_make(
-                    "model",            "",             DATA_STRING, _X("LaCrosse-TX","LaCrosse TX Sensor"),
+                    "model",            "",             DATA_STRING, "LaCrosse-TX",
                     "id",               "",             DATA_INT,    sensor_id,
                     "temperature_C",    "Temperature",  DATA_FORMAT, "%.1f C", DATA_DOUBLE, temp_c,
                     "mic",              "Integrity",    DATA_STRING, "PARITY",
@@ -171,7 +174,7 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         else if (msg_type == 0x0E) {
             /* clang-format off */
             data = data_make(
-                    "model",            "",             DATA_STRING, _X("LaCrosse-TX","LaCrosse TX Sensor"),
+                    "model",            "",             DATA_STRING, "LaCrosse-TX",
                     "id",               "",             DATA_INT,    sensor_id,
                     "humidity",         "Humidity",     DATA_FORMAT, "%.1f %%", DATA_DOUBLE, msg_value,
                     "mic",              "Integrity",    DATA_STRING, "PARITY",
@@ -190,7 +193,10 @@ static int lacrossetx_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         }
     }
 
-    return events;
+    if (events)
+      return events;
+
+    return result;
 }
 
 static char *output_fields[] = {
