@@ -14,6 +14,9 @@
 /**
 LaCrosse TX141-Bv2, TX141TH-Bv2, TX141-Bv3, TX145wsdth sensor.
 
+Also TFA 30.3221.02 (a TX141TH-Bv2),
+also TFA 30.3222.02 (a LaCrosse-TX141W).
+
 LaCrosse Color Forecast Station (model C85845), or other LaCrosse product
 utilizing the remote temperature/humidity sensor TX141TH-Bv2 transmitting
 in the 433.92 MHz band. Product pages:
@@ -45,6 +48,8 @@ The TX141TH-Bv2 sensor sends 12 of identical packets, one immediately following
 the other, in a single burst. These 12-packet bursts repeat every 50 seconds. At
 the end of the last packet there are two 833 us pulses ("post-amble"?).
 
+The TX141-Bv3 has a revision which only sends 4 packets per transmission.
+
 The data is grouped in 5 bytes / 10 nybbles
 
     [id] [id] [flags] [temp] [temp] [temp] [humi] [humi] [chk] [chk]
@@ -120,12 +125,10 @@ static int lacrosse_tx141x_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     int r;
     int device;
     uint8_t *b;
-    int type, id, battery_low, test, channel, temp_raw, humidity = 0;
-    float temp_c, speed_kmh;
 
     // Find the most frequent data packet
-    // reduce false positives, require at least 5 out of 12 repeats.
-    r = bitbuffer_find_repeated_row(bitbuffer, 5, 32); // 32
+    // reduce false positives, require at least 5 out of 12 or 3 of 4 repeats.
+    r = bitbuffer_find_repeated_row(bitbuffer, bitbuffer->num_rows > 4 ? 5 : 3, 32); // 32
     if (r < 0) {
         // try again for TX141W/TX145wsdth, require at least 2 out of 3-7 repeats.
         r = bitbuffer_find_repeated_row(bitbuffer, 2, 64); // 65
@@ -172,17 +175,17 @@ static int lacrosse_tx141x_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             return DECODE_FAIL_MIC;
         }
 
-        id          = ((b[0] & 0x07) << 16) | (b[1] << 8) | b[2];
-        battery_low = (b[3] >> 7);
-        test        = (b[3] & 0x40) >> 6;
-        channel     = (b[3] & 0x30) >> 4;
-        type        = (b[3] & 0x0f);
-        temp_raw    = (b[4] << 4) | (b[5] >> 4);
-        humidity    = ((b[5] & 0x0f) << 8) | b[6];
+        int id          = ((b[0] & 0x07) << 16) | (b[1] << 8) | b[2];
+        int battery_low = (b[3] >> 7);
+        int test        = (b[3] & 0x40) >> 6;
+        int channel     = (b[3] & 0x30) >> 4;
+        int type        = (b[3] & 0x0f);
+        int temp_raw    = (b[4] << 4) | (b[5] >> 4);
+        int humidity    = ((b[5] & 0x0f) << 8) | b[6];
 
         if (type == 1) {
             // Temp/Hum
-            temp_c = (float)temp_raw * 0.1 - 50.0;
+            float temp_c = (temp_raw - 500) * 0.1f;
 
             /* clang-format off */
             data = data_make(
@@ -199,7 +202,7 @@ static int lacrosse_tx141x_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         }
         else if (type == 2) {
             // Wind
-            speed_kmh = temp_raw * 0.1f;
+            float speed_kmh = temp_raw * 0.1f;
             // wind direction is in humidity field
 
             /* clang-format off */
@@ -226,18 +229,20 @@ static int lacrosse_tx141x_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         return 1;
     }
 
-    id = b[0];
+    int id = b[0];
+    int battery_low;
     if (device == LACROSSE_TX141TH) {
         battery_low = (b[1] >> 7);
     }
     else { // LACROSSE_TX141 || LACROSSE_TX141BV3
         battery_low = !(b[1] >> 7);
     }
-    test     = (b[1] & 0x40) >> 6;
-    channel  = (b[1] & 0x30) >> 4;
-    temp_raw = ((b[1] & 0x0F) << 8) | b[2];
-    temp_c   = (float)temp_raw * 0.1 - 50.0; // Temperature in C
+    int test     = (b[1] & 0x40) >> 6;
+    int channel  = (b[1] & 0x30) >> 4;
+    int temp_raw = ((b[1] & 0x0F) << 8) | b[2];
+    float temp_c = (temp_raw - 500) * 0.1f; // Temperature in C
 
+    int humidity = 0;
     if (device == LACROSSE_TX141TH) {
         humidity = b[3];
     }
@@ -252,21 +257,21 @@ static int lacrosse_tx141x_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     if (device == LACROSSE_TX141B) {
         /* clang-format off */
         data = data_make(
-                "model",         "",              DATA_STRING, _X("LaCrosse-TX141B","LaCrosse TX141B sensor"),
+                "model",         "",              DATA_STRING, "LaCrosse-TX141B",
                 "id",            "Sensor ID",     DATA_FORMAT, "%02x", DATA_INT, id,
                 "temperature_C", "Temperature",   DATA_FORMAT, "%.2f C", DATA_DOUBLE, temp_c,
-                "battery",       "Battery",       DATA_STRING, battery_low ? "LOW" : "OK",
+                "battery_ok",    "Battery",       DATA_INT,    !battery_low,
                 "test",          "Test?",         DATA_STRING, test ? "Yes" : "No",
                 NULL);
         /* clang-format on */
     } else if (device == LACROSSE_TX141) {
         /* clang-format off */
         data = data_make(
-                "model",         "",              DATA_STRING, _X("LaCrosse-TX141Bv2","LaCrosse TX141-Bv2 sensor"),
+                "model",         "",              DATA_STRING, "LaCrosse-TX141Bv2",
                 "id",            "Sensor ID",     DATA_FORMAT, "%02x", DATA_INT, id,
                 "channel",       "Channel",       DATA_FORMAT, "%02x", DATA_INT, channel,
                 "temperature_C", "Temperature",   DATA_FORMAT, "%.2f C", DATA_DOUBLE, temp_c,
-                "battery",       "Battery",       DATA_STRING, battery_low ? "LOW" : "OK",
+                "battery_ok",    "Battery",       DATA_INT,    !battery_low,
                 "test",          "Test?",         DATA_STRING, test ? "Yes" : "No",
                 NULL);
         /* clang-format on */
@@ -277,22 +282,27 @@ static int lacrosse_tx141x_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 "model",         "",              DATA_STRING, "LaCrosse-TX141Bv3",
                 "id",            "Sensor ID",     DATA_FORMAT, "%02x", DATA_INT, id,
                 "channel",       "Channel",       DATA_FORMAT, "%02x", DATA_INT, channel,
-                "battery",       "Battery",       DATA_STRING, battery_low ? "LOW" : "OK",
+                "battery_ok",    "Battery",       DATA_INT,    !battery_low,
                 "temperature_C", "Temperature",   DATA_FORMAT, "%.2f C", DATA_DOUBLE, temp_c,
                 "test",          "Test?",         DATA_STRING, test ? "Yes" : "No",
                 NULL);
         /* clang-format on */
     }
     else {
+        // Digest check for TX141TH-Bv2
+        if (lfsr_digest8_reflect(b, 4, 0x31, 0xf4) != b[4]) {
+            return DECODE_FAIL_MIC;
+        }
         /* clang-format off */
         data = data_make(
-                "model",         "",              DATA_STRING, _X("LaCrosse-TX141THBv2","LaCrosse TX141TH-Bv2 sensor"),
+                "model",         "",              DATA_STRING, "LaCrosse-TX141THBv2",
                 "id",            "Sensor ID",     DATA_FORMAT, "%02x", DATA_INT, id,
                 "channel",       "Channel",       DATA_FORMAT, "%02x", DATA_INT, channel,
-                "battery",       "Battery",       DATA_STRING, battery_low ? "LOW" : "OK",
+                "battery_ok",    "Battery",       DATA_INT,    !battery_low,
                 "temperature_C", "Temperature",   DATA_FORMAT, "%.2f C", DATA_DOUBLE, temp_c,
                 "humidity",      "Humidity",      DATA_FORMAT, "%u %%", DATA_INT, humidity,
                 "test",          "Test?",         DATA_STRING, test ? "Yes" : "No",
+                "mic",           "Integrity",     DATA_STRING, "CRC",
                 NULL);
         /* clang-format on */
     }
@@ -305,7 +315,6 @@ static char *output_fields[] = {
         "model",
         "id",
         "channel",
-        "battery",
         "battery_ok",
         "temperature_C",
         "humidity",
@@ -326,6 +335,5 @@ r_device lacrosse_tx141x = {
         .gap_limit   = 625,  // long gap (with short pulse) is ~417 us, sync gap is ~833 us
         .reset_limit = 1700, // maximum gap is 1250 us (long gap + longer sync gap on last repeat)
         .decode_fn   = &lacrosse_tx141x_decode,
-        .disabled    = 0,
         .fields      = output_fields,
 };
