@@ -21,10 +21,9 @@ start pulse: 1T high, 10.44T low
 
 static int newkaku_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
-    data_t *data;
-    bitrow_t *bb = bitbuffer->bb;
+    uint8_t *b = bitbuffer->bb[0];
 
-    if (bb[0][0] != 0x65 && bb[0][0] != 0x59) // always starts with 0110 0101 or 0101 1001
+    if (b[0] != 0x65 && b[0] != 0x59) // always starts with 0110 0101 or 0101 1001
         return DECODE_ABORT_EARLY;
 
     /* Reject missing sync */
@@ -36,9 +35,9 @@ static int newkaku_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         return DECODE_ABORT_LENGTH;
 
     // 11 for command indicates DIM, 4 extra bits indicate DIM value
-    uint8_t dim_cmd = (bb[0][6] & 0x03) == 0x03;
+    uint8_t dim_cmd = (b[6] & 0x03) == 0x03;
     if (dim_cmd) {
-        bb[0][6] &= 0xfe; // change DIM to ON to use Manchester
+        b[6] &= 0xfe; // change DIM to ON to use Manchester
     }
 
     bitbuffer_t databits = {0};
@@ -50,7 +49,7 @@ static int newkaku_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     if (pos != 64 && pos != 72)
         return DECODE_ABORT_LENGTH;
 
-    uint8_t *b = databits.bb[0];
+    b = databits.bb[0];
 
     uint32_t id        = (b[0] << 18) | (b[1] << 10) | (b[2] << 2) | (b[3] >> 6); // ID 26 bits
     uint32_t group_cmd = (b[3] >> 5) & 1;
@@ -59,8 +58,8 @@ static int newkaku_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     uint32_t dv        = (b[4] >> 4);
 
     /* clang-format off */
-    data = data_make(
-            "model",        "",             DATA_STRING, _X("KlikAanKlikUit-Switch","KlikAanKlikUit Wireless Switch"),
+    data_t *data = data_make(
+            "model",        "",             DATA_STRING, "KlikAanKlikUit-Switch",
             "id",           "",             DATA_INT,    id,
             "unit",         "Unit",         DATA_INT,    unit,
             "group_call",   "Group Call",   DATA_STRING, group_cmd ? "Yes" : "No",
