@@ -18,10 +18,6 @@
 
 #include "decoder.h"
 
-static const unsigned char preamble_pattern0[2] = {0x55, 0x5D};
-static const unsigned char preamble_pattern1[2] = {0x55, 0x62};
-// End of frame is the last half-bit repeated additional 4 times
-
 /**
 Oil tank monitor using manchester encoded FSK/ASK protocol.
 
@@ -87,16 +83,18 @@ static int oil_standard_decode(r_device *decoder, bitbuffer_t *bitbuffer, unsign
         // A depth reading of zero indicates no reading.
         depth = ((b[2] & 0x02) << 7) | b[3];
 
+    /* clang-format off */
     data = data_make(
-            "model", "", DATA_STRING, _X("Oil-SonicStd","Oil Ultrasonic STANDARD"),
-            "id", "", DATA_FORMAT, "%04x", DATA_INT, unit_id,
-            "flags", "", DATA_FORMAT, "%02x", DATA_INT, flags,
-            "alarm", "", DATA_INT, alarm,
-            "binding_countdown", "", DATA_INT, binding_countdown,
-            "depth_cm", "", DATA_INT, depth,
+            "model",                "", DATA_STRING, "Oil-SonicStd",
+            "id",                   "", DATA_FORMAT, "%04x", DATA_INT, unit_id,
+            "flags",                "", DATA_FORMAT, "%02x", DATA_INT, flags,
+            "alarm",                "", DATA_INT,    alarm,
+            "binding_countdown",    "", DATA_INT,    binding_countdown,
+            "depth_cm",             "", DATA_INT,    depth,
             NULL);
-    decoder_output_data(decoder, data);
+    /* clang-format on */
 
+    decoder_output_data(decoder, data);
     return 1;
 }
 
@@ -106,18 +104,22 @@ Oil tank monitor using manchester encoded FSK/ASK protocol.
 */
 static int oil_standard_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
+    uint8_t const preamble_pattern0[2] = {0x55, 0x5D};
+    uint8_t const preamble_pattern1[2] = {0x55, 0x62};
+    // End of frame is the last half-bit repeated additional 4 times
+
     unsigned bitpos = 0;
     int events = 0;
 
     // Find a preamble with enough bits after it that it could be a complete packet
-    while ((bitpos = bitbuffer_search(bitbuffer, 0, bitpos, (uint8_t *)&preamble_pattern0, 16)) + 78 <=
+    while ((bitpos = bitbuffer_search(bitbuffer, 0, bitpos, preamble_pattern0, 16)) + 78 <=
             bitbuffer->bits_per_row[0]) {
         events += oil_standard_decode(decoder, bitbuffer, 0, bitpos + 14);
         bitpos += 2;
     }
 
     bitpos = 0;
-    while ((bitpos = bitbuffer_search(bitbuffer, 0, bitpos, (uint8_t *)&preamble_pattern1, 16)) + 78 <=
+    while ((bitpos = bitbuffer_search(bitbuffer, 0, bitpos, preamble_pattern1, 16)) + 78 <=
             bitbuffer->bits_per_row[0]) {
         events += oil_standard_decode(decoder, bitbuffer, 0, bitpos + 14);
         bitpos += 2;
