@@ -17,6 +17,7 @@
 // Sensors ID
 #define ID_THGR122N 0x1d20
 #define ID_THGR968  0x1d30
+#define ID_BTHR918  0x5d50
 #define ID_BHTR968  0x5d60
 #define ID_RGR968   0x2d10
 #define ID_THR228N  0xec40
@@ -113,6 +114,32 @@ static unsigned int get_os_rollingcode(unsigned char *message)
     return rc;
 }
 
+static unsigned short int cm180i_power(uint8_t const *msg,unsigned int offset)
+{
+    unsigned short int val = 0;
+
+    val = (msg[4+offset*2] << 8) | (msg[3+offset*2] & 0xF0);
+    // tested accross situations varying from 700 watt to more than 8000 watt to
+    // get same value as showed in physical CM180 panel (exactly equals to 1+1/160)
+    val *= 1.00625;
+    return val;
+}
+
+static unsigned long long cm180i_total(uint8_t const *msg)
+{
+    unsigned long long val = 0;
+    if ((msg[1] & 0x0F) == 0) {
+        // Sensor returns total only if nibble#4 == 0
+        val = (unsigned long long)msg[14] << 40;
+        val += (unsigned long long)msg[13] << 32;
+        val += (unsigned long)msg[12] << 24;
+        val += (unsigned long)msg[11] << 16;
+        val += msg[10] << 8;
+        val += msg[9];
+    }
+    return val;
+}
+
 static unsigned short int cm180_power(uint8_t const *msg)
 {
     unsigned short int val = 0;
@@ -238,15 +265,16 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
     if ((sensor_id == ID_THGR122N) || (sensor_id == ID_THGR968)) {
         if (validate_os_v2_message(decoder, msg, 76, msg_bits, 15) != 0)
             return 0;
+        /* clang-format off */
         data = data_make(
-                "brand",                 "",                        DATA_STRING, "OS",
-                "model",                 "",                        DATA_STRING, (sensor_id == ID_THGR122N) ? _X("Oregon-THGR122N","THGR122N"): _X("Oregon-THGR968","THGR968"),
+                "model",                 "",                        DATA_STRING, (sensor_id == ID_THGR122N) ? "Oregon-THGR122N" : "Oregon-THGR968",
                 "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                 "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C", "Temperature", DATA_FORMAT, "%.02f C", DATA_DOUBLE, get_os_temperature(msg),
                 "humidity",            "Humidity",        DATA_FORMAT, "%u %%",     DATA_INT,        get_os_humidity(msg),
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -256,16 +284,17 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         float quadrant      = (msg[4] & 0x0f) * 10 + ((msg[4] >> 4) & 0x0f) * 1 + ((msg[5] >> 4) & 0x0f) * 100;
         float avgWindspeed  = ((msg[7] >> 4) & 0x0f) / 10.0F + (msg[7] & 0x0f) * 1.0F + ((msg[8] >> 4) & 0x0f) / 10.0F;
         float gustWindspeed = (msg[5] & 0x0f) / 10.0F + ((msg[6] >> 4) & 0x0f) * 1.0F + (msg[6] & 0x0f) / 10.0F;
+        /* clang-format off */
         data = data_make(
-                "brand",            "",                     DATA_STRING, "OS",
-                "model",            "",                     DATA_STRING, _X("Oregon-WGR968","WGR968"),
+                "model",            "",                     DATA_STRING, "Oregon-WGR968",
                 "id",                 "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",        "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",        "Battery",        DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
-                _X("wind_max_m_s","gust"), "Gust",             DATA_FORMAT, "%2.1f m/s",DATA_DOUBLE, gustWindspeed,
-                _X("wind_avg_m_s","average"), "Average",        DATA_FORMAT, "%2.1f m/s",DATA_DOUBLE, avgWindspeed,
-                _X("wind_dir_deg","direction"),    "Direction",    DATA_FORMAT, "%3.1f degrees",DATA_DOUBLE, quadrant,
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
+                "wind_max_m_s", "Gust",             DATA_FORMAT, "%2.1f m/s",DATA_DOUBLE, gustWindspeed,
+                "wind_avg_m_s", "Average",        DATA_FORMAT, "%2.1f m/s",DATA_DOUBLE, avgWindspeed,
+                "wind_dir_deg",    "Direction",    DATA_FORMAT, "%3.1f degrees",DATA_DOUBLE, quadrant,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -286,16 +315,37 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         float pressure = ((msg[7] & 0x0f) | (msg[8] & 0xf0)) + 856;
         // fprintf(stderr,"Weather Sensor BHTR968    Indoor        Temp: %3.1fC    %3.1fF     Humidity: %d%%", temp_c, ((temp_c*9)/5)+32, get_os_humidity(msg));
         // fprintf(stderr, " (%s) Pressure: %dmbar (%s)\n", comfort_str, ((msg[7] & 0x0f) | (msg[8] & 0xf0))+856, forecast_str);
+        /* clang-format off */
         data = data_make(
-                "brand",            "",                             DATA_STRING, "OS",
-                "model",            "",                             DATA_STRING, _X("Oregon-BHTR968","BHTR968"),
+                "model",            "",                             DATA_STRING, "Oregon-BHTR968",
                 "id",                 "House Code",         DATA_INT,        get_os_rollingcode(msg),
                 "channel",        "Channel",                DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",        "Battery",                DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 "humidity",     "Humidity",             DATA_FORMAT, "%u %%",     DATA_INT,        get_os_humidity(msg),
                 "pressure_hPa",    "Pressure",        DATA_FORMAT, "%.0f hPa",     DATA_DOUBLE, pressure,
                 NULL);
+        /* clang-format on */
+        decoder_output_data(decoder, data);
+        return 1;
+    }
+    else if (sensor_id == ID_BTHR918) {
+        // Similar to the BHTR968, but smaller message and slightly different pressure offset
+        if (validate_os_v2_message(decoder, msg, 84, msg_bits, 19) != 0)
+            return 0;
+        float temp_c = get_os_temperature(msg);
+        float pressure = ((msg[7] & 0x0f) | (msg[8] & 0xf0)) + 795;
+        /* clang-format off */
+        data = data_make(
+                "model",            "",                 DATA_STRING,    "Oregon-BTHR918",
+                "id",               "House Code",       DATA_INT,       get_os_rollingcode(msg),
+                "channel",          "Channel",          DATA_INT,       get_os_channel(msg, sensor_id),
+                "battery_ok",       "Battery",          DATA_INT,       !get_os_battery(msg),
+                "temperature_C",    "Celsius",          DATA_FORMAT,    "%.02f C", DATA_DOUBLE, temp_c,
+                "humidity",         "Humidity",         DATA_FORMAT,    "%u %%", DATA_INT, get_os_humidity(msg),
+                "pressure_hPa",     "Pressure",         DATA_FORMAT,    "%.0f hPa", DATA_DOUBLE, pressure,
+                NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -304,15 +354,16 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
             return 0;
         float rain_rate  = ((msg[4] & 0x0f) * 100 + (msg[4] >> 4) * 10 + ((msg[5] >> 4) & 0x0f)) / 10.0F;
         float total_rain = ((msg[7] & 0xf) * 10000 + (msg[7] >> 4) * 1000 + (msg[6] & 0xf) * 100 + (msg[6] >> 4) * 10 + (msg[5] & 0xf)) / 10.0F;
+        /* clang-format off */
         data = data_make(
-                "brand",            "",                     DATA_STRING, "OS",
-                "model",            "",                     DATA_STRING, _X("Oregon-RGR968","RGR968"),
+                "model",            "",                     DATA_STRING, "Oregon-RGR968",
                 "id",                 "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",        "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",        "Battery",        DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
-                _X("rain_rate_mm_h","rain_rate"),    "Rain Rate",    DATA_FORMAT, "%.02f mm/h", DATA_DOUBLE, rain_rate,
-                _X("rain_mm","total_rain"), "Total Rain", DATA_FORMAT, "%.02f mm", DATA_DOUBLE, total_rain,
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
+                "rain_rate_mm_h",    "Rain Rate",    DATA_FORMAT, "%.02f mm/h", DATA_DOUBLE, rain_rate,
+                "rain_mm", "Total Rain", DATA_FORMAT, "%.02f mm", DATA_DOUBLE, total_rain,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -320,14 +371,15 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         if (validate_os_v2_message(decoder, msg, 76, msg_bits, 12) != 0)
             return 0;
         float temp_c = get_os_temperature(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",                 "",                        DATA_STRING, "OS",
-                "model",                 "",                        DATA_STRING, _X("Oregon-THR228N","THR228N"),
+                "model",                 "",                        DATA_STRING, "Oregon-THR228N",
                 "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                 "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -335,14 +387,15 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         if (validate_os_v2_message(decoder, msg, 64, msg_bits, 12) != 0)
             return 0;
         float temp_c = get_os_temperature(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",                 "",                        DATA_STRING, "OS",
-                "model",                 "",                        DATA_STRING, _X("Oregon-THN132N","THN132N"),
+                "model",                 "",                        DATA_STRING, "Oregon-THN132N",
                 "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                 "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -350,30 +403,32 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         if (validate_os_v2_message(decoder, msg, 80, msg_bits, 15) != 0)
             return 0;
         float temp_c = get_os_temperature(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",                 "",                        DATA_STRING, "OS",
-                "model",                 "",                        DATA_STRING, _X("Oregon-RTGN129","RTGN129"),
+                "model",                 "",                        DATA_STRING, "Oregon-RTGN129",
                 "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                 "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id), // 1 to 5
-                "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 "humidity",            "Humidity",        DATA_FORMAT, "%u %%",     DATA_INT,        get_os_humidity(msg),
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
     else if (((sensor_id == ID_RTGR328N_1) || (sensor_id == ID_RTGR328N_2) || (sensor_id == ID_RTGR328N_3) || (sensor_id == ID_RTGR328N_4) || (sensor_id == ID_RTGR328N_5)) && msg_bits == 173) {
         if (validate_os_v2_message(decoder, msg, 173, msg_bits, 15) != 0)
              return 0;
+        /* clang-format off */
         data = data_make(
-                "brand",            "",             DATA_STRING, "OS",
                 "model",            "",             DATA_STRING, "Oregon-RTGR328N",
                 "id",               "House Code",   DATA_INT,    get_os_rollingcode(msg),
                 "channel",          "Channel",      DATA_INT,    get_os_channel(msg, sensor_id), // 1 to 5
-                "battery",          "Battery",      DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Temperature",  DATA_FORMAT, "%.02f C", DATA_DOUBLE, get_os_temperature(msg),
                 "humidity",         "Humidity",     DATA_FORMAT, "%u %%",   DATA_INT,    get_os_humidity(msg),
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -393,29 +448,31 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         sprintf(clock_str, "%04d-%02d-%02dT%02d:%02d:%02d",
                 year, month, day, hours, minutes, seconds);
 
+        /* clang-format off */
         data = data_make(
-                "brand",            "",             DATA_STRING, "OS",
                 "model",            "",             DATA_STRING, "Oregon-RTGR328N",
                 "id",               "House Code",   DATA_INT,    get_os_rollingcode(msg),
                 "channel",          "Channel",      DATA_INT,    get_os_channel(msg, sensor_id), // 1 to 5
-                "battery",          "Battery",      DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "radio_clock",      "Radio Clock",  DATA_STRING, clock_str,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
     else if ((sensor_id & 0x0fff) == ID_RTGN318) {
         if (msg_bits == 76 && (validate_os_v2_message(decoder, msg, 76, msg_bits, 15) == 0)) {
             float temp_c = get_os_temperature(msg);
+            /* clang-format off */
             data = data_make(
-                    "brand",                 "",                        DATA_STRING, "OS",
-                    "model",                 "",                        DATA_STRING, _X("Oregon-RTGN318","RTGN318"),
+                    "model",                 "",                        DATA_STRING, "Oregon-RTGN318",
                     "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                     "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id), // 1 to 5
-                    "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                    "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                     "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                     "humidity",            "Humidity",        DATA_FORMAT, "%u %%",     DATA_INT,        get_os_humidity(msg),
                     NULL);
+            /* clang-format on */
             decoder_output_data(decoder, data);
             return 1;
         }
@@ -427,14 +484,15 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
     else if (sensor_id == ID_THN129 || (sensor_id & 0x0FFF) == ID_RTHN129) {
         if ((validate_os_v2_message(decoder, msg, 68, msg_bits, 12) == 0)) {
             float temp_c = get_os_temperature(msg);
+            /* clang-format off */
             data = data_make(
-                    "brand",                 "",                        DATA_STRING, "OS",
-                    "model",                 "",                        DATA_STRING, (sensor_id == ID_THN129) ? _X("Oregon-THN129","THN129") : "Oregon-RTHN129",
+                    "model",                 "",                        DATA_STRING, (sensor_id == ID_THN129) ? "Oregon-THN129" : "Oregon-RTHN129",
                     "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                     "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id), // 1 to 5
-                    "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                     "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                     NULL);
+            /* clang-format on */
             decoder_output_data(decoder, data);
             return 1;
         }
@@ -450,16 +508,17 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         // Pressure is given in hPa. You may need to adjust the offset
         // according to your altitude level (600 is a good starting point)
         float pressure = ((msg[7] & 0x0f) | (msg[8] & 0xf0)) * 2 + (msg[8] & 0x01) + 600;
+        /* clang-format off */
         data = data_make(
-                "brand",                 "",                        DATA_STRING, "OS",
-                "model",                 "",                        DATA_STRING, _X("Oregon-BTHGN129","BTHGN129"),
+                "model",                 "",                        DATA_STRING, "Oregon-BTHGN129",
                 "id",                        "House Code",    DATA_INT,        get_os_rollingcode(msg),
                 "channel",             "Channel",         DATA_INT,        get_os_channel(msg, sensor_id), // 1 to 5
-                "battery",             "Battery",         DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 "humidity",             "Humidity",     DATA_FORMAT, "%u %%", DATA_INT, get_os_humidity(msg),
                 "pressure_hPa",    "Pressure",        DATA_FORMAT, "%.02f hPa", DATA_DOUBLE, pressure,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -467,28 +526,31 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
         if (validate_os_v2_message(decoder, msg, 148, msg_bits, 12) != 0)
             return 0;
         int uvidx = get_os_uv(msg);
+        /* clang-format off */
         data = data_make(
-                "model",                    "",                     DATA_STRING, _X("Oregon-UVR128","Oregon Scientific UVR128"),
+                "model",                    "",                     DATA_STRING, "Oregon-UVR128",
                 "id",                         "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "uv",                         "UV Index",     DATA_FORMAT, "%u", DATA_INT, uvidx,
-                "battery",                "Battery",        DATA_STRING, get_os_battery(msg)?"LOW":"OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 //"channel",                "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
     else if (sensor_id == ID_THGR328N) {
         if (validate_os_v2_message(decoder, msg, 173, msg_bits, 15) != 0)
             return 0;
+        /* clang-format off */
         data = data_make(
-                "brand",            "",             DATA_STRING, "OS",
                 "model",            "",             DATA_STRING, "Oregon-THGR328N",
                 "id",               "House Code",   DATA_INT,    get_os_rollingcode(msg),
                 "channel",          "Channel",      DATA_INT,    get_os_channel(msg, sensor_id), // 1 to 5
-                "battery",          "Battery",      DATA_STRING, get_os_battery(msg) ? "LOW" : "OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Temperature",  DATA_FORMAT, "%.02f C", DATA_DOUBLE, get_os_temperature(msg),
                 "humidity",         "Humidity",     DATA_FORMAT, "%u %%",   DATA_INT,    get_os_humidity(msg),
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -506,6 +568,9 @@ static int oregon_scientific_v2_1_decode(r_device *decoder, bitbuffer_t *bitbuff
     return 0;
 }
 
+// ceil( (335 + 11) / 8 )
+#define EXPECTED_NUM_BYTES 44
+
 static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
     uint8_t *b = bitbuffer->bb[0];
@@ -521,7 +586,7 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
         return DECODE_ABORT_EARLY;
     }
 
-    unsigned char msg[BITBUF_COLS] = {0};
+    unsigned char msg[EXPECTED_NUM_BYTES] = {0};
     int msg_pos = 0;
     int msg_len = 0;
 
@@ -533,12 +598,14 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
     uint8_t const os_pattern[] = {0x00, 0x05};
     // CM180 preamble is 00 00 00 46, with 0x46 already data
     uint8_t const cm180_pattern[] = {0x00, 0x46};
+    uint8_t const cm180i_pattern[] = {0x00, 0x4A};
     // workaround for a broken manchester demod
     // CM160 preamble might look like 7f ff ff aa, i.e. ff ff f5
     uint8_t const alt_pattern[] = {0xff, 0xf5};
 
     int os_pos    = bitbuffer_search(bitbuffer, 0, 0, os_pattern, 16) + 16;
     int cm180_pos = bitbuffer_search(bitbuffer, 0, 0, cm180_pattern, 16) + 8; // keep the 0x46
+    int cm180i_pos = bitbuffer_search(bitbuffer, 0, 0, cm180i_pattern, 16) + 8; // keep the 0x46
     int alt_pos   = bitbuffer_search(bitbuffer, 0, 0, alt_pattern, 16) + 16;
 
     if (bitbuffer->bits_per_row[0] - os_pos >= 7 * 8) {
@@ -553,12 +620,17 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
         msg_len = bitbuffer->bits_per_row[0] - cm180_pos;
     }
 
+    else if (bitbuffer->bits_per_row[0] - cm180i_pos >= 84) {
+        msg_pos = cm180i_pos;
+        msg_len = bitbuffer->bits_per_row[0] - cm180i_pos;
+    }
+
     else if (bitbuffer->bits_per_row[0] - alt_pos >= 7 * 8) {
         msg_pos = alt_pos;
         msg_len = bitbuffer->bits_per_row[0] - alt_pos;
     }
 
-    if (msg_len == 0)
+    if (msg_len == 0 || msg_len > (int)sizeof(msg) * 8)
         return DECODE_ABORT_EARLY;
 
     bitbuffer_extract_bytes(bitbuffer, 0, msg_pos, msg, msg_len);
@@ -570,15 +642,16 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
             return DECODE_FAIL_MIC;
         float temp_c = get_os_temperature(msg);
         int humidity = get_os_humidity(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",                    "",                     DATA_STRING, "OS",
-                "model",                    "",                     DATA_STRING, _X("Oregon-THGR810","THGR810"),
+                "model",                    "",                     DATA_STRING, "Oregon-THGR810",
                 "id",                         "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",                "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",                "Battery",        DATA_STRING, get_os_battery(msg)?"LOW":"OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 "humidity",             "Humidity",     DATA_FORMAT, "%u %%", DATA_INT, humidity,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;                                    //msg[k] = ((msg[k] & 0x0F) << 4) + ((msg[k] & 0xF0) >> 4);
     }
@@ -586,14 +659,15 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
         if (validate_os_checksum(decoder, msg, 12) != 0)
             return DECODE_FAIL_MIC;
         float temp_c = get_os_temperature(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",                    "",                     DATA_STRING, "OS",
-                "model",                    "",                     DATA_STRING, _X("Oregon-THN802","THN802"),
+                "model",                    "",                     DATA_STRING, "Oregon-THN802",
                 "id",                         "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",                "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",                "Battery",        DATA_STRING, get_os_battery(msg)?"LOW":"OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "temperature_C",    "Celsius",        DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -601,14 +675,15 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
         if (validate_os_checksum(decoder, msg, 13) != 0)
             return DECODE_FAIL_MIC;
         int uvidx = get_os_uv(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",                    "",                     DATA_STRING, "OS",
-                "model",                    "",                     DATA_STRING, _X("Oregon-UV800","UV800"),
+                "model",                    "",                     DATA_STRING, "Oregon-UV800",
                 "id",                         "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",                "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",                "Battery",        DATA_STRING, get_os_battery(msg)?"LOW":"OK",
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
                 "uv",                         "UV Index",     DATA_FORMAT, "%u", DATA_INT, uvidx,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -617,15 +692,16 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
             return DECODE_FAIL_MIC;
         float rain_rate = get_os_rain_rate(msg);
         float total_rain = get_os_total_rain(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",            "",                     DATA_STRING, "OS",
-                "model",            "",                     DATA_STRING, _X("Oregon-PCR800","PCR800"),
+                "model",            "",                     DATA_STRING, "Oregon-PCR800",
                 "id",                 "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",        "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",        "Battery",        DATA_STRING, get_os_battery(msg)?"LOW":"OK",
-                _X("rain_rate_in_h","rain_rate"),    "Rain Rate",    DATA_FORMAT, "%3.1f in/h", DATA_DOUBLE, rain_rate,
-                _X("rain_in","rain_total"), "Total Rain", DATA_FORMAT, "%3.1f in", DATA_DOUBLE, total_rain,
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
+                "rain_rate_in_h",    "Rain Rate",    DATA_FORMAT, "%3.1f in/h", DATA_DOUBLE, rain_rate,
+                "rain_in", "Total Rain", DATA_FORMAT, "%3.1f in", DATA_DOUBLE, total_rain,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -634,15 +710,16 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
             return DECODE_FAIL_MIC;
         float rain_rate = get_os_rain_rate(msg);
         float total_rain = get_os_total_rain(msg);
+        /* clang-format off */
         data = data_make(
-                "brand",            "",                     DATA_STRING, "OS",
-                "model",            "",                     DATA_STRING, _X("Oregon-PCR800a","PCR800a"),
+                "model",            "",                     DATA_STRING, "Oregon-PCR800a",
                 "id",                 "House Code", DATA_INT,        get_os_rollingcode(msg),
                 "channel",        "Channel",        DATA_INT,        get_os_channel(msg, sensor_id),
-                "battery",        "Battery",        DATA_STRING, get_os_battery(msg)?"LOW":"OK",
-                _X("rain_rate_in_h","rain_rate"),    "Rain Rate",    DATA_FORMAT, "%3.1f in/h", DATA_DOUBLE, rain_rate,
-                _X("rain_in","rain_total"), "Total Rain", DATA_FORMAT, "%3.1f in", DATA_DOUBLE, total_rain,
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
+                "rain_rate_in_h",    "Rain Rate",    DATA_FORMAT, "%3.1f in/h", DATA_DOUBLE, rain_rate,
+                "rain_in", "Total Rain", DATA_FORMAT, "%3.1f in", DATA_DOUBLE, total_rain,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -652,16 +729,17 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
         float gustWindspeed = (msg[5]&0x0f) /10.0F + ((msg[6]>>4)&0x0f) *1.0F + (msg[6]&0x0f) * 10.0F;
         float avgWindspeed = ((msg[7]>>4)&0x0f) / 10.0F + (msg[7]&0x0f) *1.0F + ((msg[8]>>4)&0x0f) * 10.0F;
         float quadrant = (0x0f&(msg[4]>>4))*22.5F;
+        /* clang-format off */
         data = data_make(
-                "brand",            "",                     DATA_STRING,    "OS",
-                "model",            "",                     DATA_STRING,    _X("Oregon-WGR800","WGR800"),
+                "model",            "",                     DATA_STRING,    "Oregon-WGR800",
                 "id",                 "House Code", DATA_INT,         get_os_rollingcode(msg),
                 "channel",        "Channel",        DATA_INT,         get_os_channel(msg, sensor_id),
-                "battery",        "Battery",        DATA_STRING,    get_os_battery(msg)?"LOW":"OK",
-                _X("wind_max_m_s","gust"),             "Gust",             DATA_FORMAT,    "%2.1f m/s",DATA_DOUBLE, gustWindspeed,
-                _X("wind_avg_m_s","average"),        "Average",        DATA_FORMAT,    "%2.1f m/s",DATA_DOUBLE, avgWindspeed,
-                _X("wind_dir_deg","direction"),    "Direction",    DATA_FORMAT,    "%3.1f degrees",DATA_DOUBLE, quadrant,
+                "battery_ok",          "Battery",         DATA_INT,    !get_os_battery(msg),
+                "wind_max_m_s",             "Gust",             DATA_FORMAT,    "%2.1f m/s",DATA_DOUBLE, gustWindspeed,
+                "wind_avg_m_s",        "Average",        DATA_FORMAT,    "%2.1f m/s",DATA_DOUBLE, avgWindspeed,
+                "wind_dir_deg",    "Direction",    DATA_FORMAT,    "%3.1f degrees",DATA_DOUBLE, quadrant,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -671,19 +749,20 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
             return DECODE_FAIL_MIC;
         float rawAmp = (msg[4] >> 4 << 8 | (msg[3] & 0x0f )<< 4 | msg[3] >> 4);
         unsigned short int ipower = (rawAmp /(0.27*230)*1000);
+        /* clang-format off */
         data = data_make(
-                "brand",    "",                     DATA_STRING, "OS",
-                "model",    "",                     DATA_STRING,    _X("Oregon-CM160","CM160"),
+                "model",    "",                     DATA_STRING,    "Oregon-CM160",
                 "id",         "House Code", DATA_INT, msg[1]&0x0F,
                 "power_W", "Power",         DATA_FORMAT,    "%d W", DATA_INT, ipower,
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
     else if (msg[0] == 0x26) { // Owl CM180 readings
         msg[0]    = msg[0] & 0x0f;
         int valid = validate_os_checksum(decoder, msg, 23);
-        for (int k = 0; k < BITBUF_COLS; k++) { // Reverse nibbles
+        for (int k = 0; k < EXPECTED_NUM_BYTES; k++) { // Reverse nibbles
             msg[k] = (msg[k] & 0xF0) >> 4 | (msg[k] & 0x0F) << 4;
         }
         // TODO: should we return if valid == 0?
@@ -695,28 +774,57 @@ static int oregon_scientific_v3_decode(r_device *decoder, bitbuffer_t *bitbuffer
         unsigned short int ipower = cm180_power(msg);
         unsigned long long itotal = cm180_total(msg);
         float total_energy        = itotal / 3600.0 / 1000.0;
-        if (itotal && valid == 0) {
+        if (valid == 0) {
+            /* clang-format off */
             data = data_make(
-                    "brand",            "",                 DATA_STRING, "OS",
-                    "model",            "",                 DATA_STRING, _X("Oregon-CM180","CM180"),
+                    "model",            "",                 DATA_STRING, "Oregon-CM180",
                     "id",               "House Code",       DATA_INT,    id,
-                    "battery",          "Battery",          DATA_STRING, batt_low ? "LOW" : "OK",
+                    "battery_ok",       "Battery",          DATA_INT,    !batt_low,
                     "power_W",          "Power",            DATA_FORMAT, "%d W",DATA_INT, ipower,
-                    "energy_kWh",       "Energy",           DATA_FORMAT, "%2.2f kWh",DATA_DOUBLE, total_energy,
+                    "energy_kWh",       "Energy",           DATA_COND,   itotal != 0, DATA_FORMAT, "%2.2f kWh",DATA_DOUBLE, total_energy,
                     "sequence",         "sequence number",  DATA_INT,    sequence,
                     NULL);
+            /* clang-format on */
             decoder_output_data(decoder, data);
             return 1;
         }
-        else if (!itotal) {
+    }
+    else if (msg[0] == 0x25) { // Owl CM180i readings
+        int valid = 0;
+        msg[0]    = msg[0] & 0x0f;
+        // to be done
+        // int valid = validate_os_checksum(decoder, msg, 23);
+        for (int k = 0; k < EXPECTED_NUM_BYTES; k++) { // Reverse nibbles
+            msg[k] = (msg[k] & 0xF0) >> 4 | (msg[k] & 0x0F) << 4;
+        }
+        // TODO: should we return if valid == 0?
+
+        int sequence = msg[1] & 0x0F;
+        int id       = msg[2] << 8 | (msg[1] & 0xF0);
+        int batt_low = (msg[3] & 0x40)?1:0; // 8th bit instead of 6th commonly used for other devices
+
+        unsigned short int ipower1 = cm180i_power(msg,0);
+        unsigned short int ipower2 = cm180i_power(msg,1);
+        unsigned short int ipower3 = cm180i_power(msg,2);
+        unsigned long long itotal= 0;
+        if (msg_len >= 140) itotal= cm180i_total(msg);
+
+        // per hour and in kilowat
+        float total_energy        = itotal / 3600.0 / 1000.0;
+
+        if (valid == 0) {
+            /* clang-format off */
             data = data_make(
-                    "brand",            "",                 DATA_STRING, "OS",
-                    "model",            "",                 DATA_STRING, _X("Oregon-CM180","CM180"),
+                    "model",            "",                 DATA_STRING, "Oregon-CM180i",
                     "id",               "House Code",       DATA_INT,    id,
-                    "battery",          "Battery",          DATA_STRING, batt_low ? "LOW" : "OK",
-                    "power_W",          "Power",            DATA_FORMAT, "%d W",DATA_INT, ipower,
+                    "battery_ok",       "Battery",          DATA_INT,    !batt_low,
+                    "power1_W",         "Power1",           DATA_FORMAT, "%d W",DATA_INT, ipower1,
+                    "power2_W",         "Power2",           DATA_FORMAT, "%d W",DATA_INT, ipower2,
+                    "power3_W",         "Power3",           DATA_FORMAT, "%d W",DATA_INT, ipower3,
+                    "energy_kWh",       "Energy",           DATA_COND,   itotal != 0, DATA_FORMAT, "%2.2f kWh",DATA_DOUBLE, total_energy,
                     "sequence",         "sequence number",  DATA_INT,    sequence,
                     NULL);
+            /* clang-format on */
             decoder_output_data(decoder, data);
             return 1;
         }
@@ -750,11 +858,10 @@ static int oregon_scientific_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 }
 
 static char *output_fields[] = {
-        "brand",
         "model",
         "id",
         "channel",
-        "battery",
+        "battery_ok",
         "temperature_C",
         "humidity",
         "rain_rate", // TODO: remove this
