@@ -1,9 +1,18 @@
-/* RadioHead ASK (generic) protocol
- *
- * Default transmitter speed is 2000 bits per second, i.e. 500 us per bit.
- * The symbol encoding ensures a maximum run (gap) of 4x bit-width.
- * Sensible Living uses a speed of 1000, i.e. 1000 us per bit.
- */
+/** @file
+    RadioHead ASK (generic) protocol.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+*/
+/**
+RadioHead ASK (generic) protocol.
+
+Default transmitter speed is 2000 bits per second, i.e. 500 us per bit.
+The symbol encoding ensures a maximum run (gap) of 4x bit-width.
+Sensible Living uses a speed of 1000, i.e. 1000 us per bit.
+*/
 
 #include "decoder.h"
 
@@ -163,7 +172,7 @@ static int radiohead_ask_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     }
     /* clang-format off */
     data = data_make(
-            "model",        "",             DATA_STRING, _X("RadioHead-ASK","RadioHead ASK"),
+            "model",        "",             DATA_STRING, "RadioHead-ASK",
             "len",          "Data len",     DATA_INT, data_len,
             "to",           "To",           DATA_INT, header_to,
             "from",         "From",         DATA_INT, header_from,
@@ -202,14 +211,14 @@ static int sensible_living_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     /* clang-format off */
     data = data_make(
-            "model",            "",                 DATA_STRING,  _X("SensibleLiving-Moisture","Sensible Living Plant Moisture"),
+            "model",            "",                 DATA_STRING,  "SensibleLiving-Moisture",
             "house_id",         "House ID",         DATA_INT,     house_id,
             "module_id",        "Module ID",        DATA_INT,     module_id,
             "sensor_type",      "Sensor Type",      DATA_INT,     sensor_type,
             "sensor_count",     "Sensor Count",     DATA_INT,     sensor_count,
             "alarms",           "Alarms",           DATA_INT,     alarms,
             "sensor_value",     "Sensor Value",     DATA_INT,     sensor_value,
-            _X("battery_mV","battery_voltage"),       "Battery Voltage",  DATA_INT,     _X(battery_voltage * 10, battery_voltage),
+            "battery_mV",       "Battery Voltage",  DATA_INT,     battery_voltage * 10,
             "mic",              "Integrity",        DATA_STRING,  "CRC",
             NULL);
     /* clang-format on */
@@ -238,7 +247,6 @@ static char *sensible_living_output_fields[] = {
     "sensor_count",
     "alarms",
     "sensor_value",
-    "battery_voltage", // TODO: remove this
     "battery_mV",
     "mic",
     NULL,
