@@ -211,6 +211,27 @@ static void print_influx_string_escaped(data_output_t *output, char const *str,
     *buf++ = '"';
     size--;
     for (; *str && size >= 3; ++str) {
+        if (*str == '\r') {
+            *buf++ = '\\';
+            size--;
+            *buf++ = 'r';
+            size--;
+            continue;
+        }
+        if (*str == '\n') {
+            *buf++ = '\\';
+            size--;
+            *buf++ = 'n';
+            size--;
+            continue;
+        }
+        if (*str == '\t') {
+            *buf++ = '\\';
+            size--;
+            *buf++ = 't';
+            size--;
+            continue;
+        }
         if (*str == '"' || *str == '\\') {
             *buf++ = '\\';
             size--;
@@ -277,11 +298,11 @@ static void print_influx_data(data_output_t *output, data_t *data, char const *f
                 || !strcmp(data->key, "time")) {
             // skip
         }
-        else if (!strcmp(data->key, "brand")
-                || !strcmp(data->key, "type")
+        else if (!strcmp(data->key, "type")
                 || !strcmp(data->key, "subtype")
                 || !strcmp(data->key, "id")
-                || !strcmp(data->key, "channel")) {
+                || !strcmp(data->key, "channel")
+                || !strcmp(data->key, "mic")) {
             str = mbuf_snprintf(buf, ",%s=", data->key);
             str++;
             end = &buf->buf[buf->len - 1];
@@ -306,11 +327,11 @@ static void print_influx_data(data_output_t *output, data_t *data, char const *f
                 || !strcmp(data->key, "time")) {
             // skip
         }
-        else if (!strcmp(data->key, "brand")
-                || !strcmp(data->key, "type")
+        else if (!strcmp(data->key, "type")
                 || !strcmp(data->key, "subtype")
                 || !strcmp(data->key, "id")
-                || !strcmp(data->key, "channel")) {
+                || !strcmp(data->key, "channel")
+                || !strcmp(data->key, "mic")) {
             // skip
         }
         else {
