@@ -44,7 +44,7 @@ static int account_event(r_device *device, bitbuffer_t *bits, char const *demod_
     }
 
     // Debug printout
-    if (!device->decode_fn || (device->verbose && ret > 0)) {
+    if (!device->decode_fn || (device->verbose && ret > 0) || device->verbose > 1) {
         fprintf(stderr, "%s(): %s\n", demod_name, device->name);
         bitbuffer_print(bits);
     }
@@ -87,6 +87,7 @@ int pulse_demod_pcm(const pulse_data_t *pulses, r_device *device)
 
     // if there is a run of bit-wide toggles (preamble) tune the bit period
     int min_count = s_short == s_long ? 12 : 4;
+    int preamble_len = 0;
     // RZ
     for (unsigned n = 0; s_short != s_long && n < pulses->num_pulses; ++n) {
         int swidth = 0;
@@ -107,6 +108,7 @@ int pulse_demod_pcm(const pulse_data_t *pulses, r_device *device)
             f_long  = (float)count / lwidth;
             f_short = (float)count / swidth;
             min_count = count;
+            preamble_len = count;
             if (device->verbose > 1) {
                 float to_us = 1e6 / pulses->sample_rate;
                 fprintf(stderr, "Exact bit width (in us) is %.2f vs %.2f (pulse width %.2f vs %.2f), %d bit preamble\n",
@@ -115,6 +117,31 @@ int pulse_demod_pcm(const pulse_data_t *pulses, r_device *device)
             }
         }
     }
+    // RZ bits within tolerance anywhere
+    int rzs_width = 0;
+    int rzl_width = 0;
+    int rz_count = 0;
+    for (unsigned n = 0; preamble_len == 0 && s_short != s_long && n < pulses->num_pulses; ++n) {
+        if (pulses->pulse[n] >= s_short - s_tolerance
+                && pulses->pulse[n] <= s_short + s_tolerance
+                && pulses->pulse[n] + pulses->gap[n] >= s_long - s_tolerance
+                && pulses->pulse[n] + pulses->gap[n] <= s_long + s_tolerance) {
+            rzs_width += pulses->pulse[n];
+            rzl_width += pulses->pulse[n] + pulses->gap[n];
+            rz_count += 1;
+        }
+    }
+    // require at least 8 bits measured
+    if (rz_count > 8) {
+        f_long  = (float)rz_count / rzl_width;
+        f_short = (float)rz_count / rzs_width;
+        if (device->verbose > 1) {
+            float to_us = 1e6 / pulses->sample_rate;
+            fprintf(stderr, "Exact bit width (in us) is %.2f vs %.2f (pulse width %.2f vs %.2f), %d bit measured\n",
+                    to_us / f_long, to_us * s_long,
+                    to_us / f_short, to_us * s_short, rz_count);
+        }
+    }
     // NRZ
     for (unsigned n = 0; s_short == s_long && n < pulses->num_pulses; ++n) {
         int width = 0;
@@ -130,6 +157,7 @@ int pulse_demod_pcm(const pulse_data_t *pulses, r_device *device)
         if (count >= min_count) {
             f_short = f_long = (float)count / width;
             min_count = count;
+            preamble_len = count;
             if (device->verbose > 1) {
                 float to_us = 1e6 / pulses->sample_rate;
                 fprintf(stderr, "Exact bit width (in us) is %.2f vs %.2f, %d bit preamble\n",
@@ -137,6 +165,40 @@ int pulse_demod_pcm(const pulse_data_t *pulses, r_device *device)
             }
         }
     }
+    // NRZ pulse/gap of len 1 or 2 within tolerance anywhere
+    int nrz_width = 0;
+    int nrz_count = 0;
+    for (unsigned n = 0; preamble_len == 0 && s_short == s_long && n < pulses->num_pulses; ++n) {
+        if (pulses->pulse[n] >= s_short - s_tolerance
+                && pulses->pulse[n] <= s_short + s_tolerance) {
+            nrz_width += pulses->pulse[n];
+            nrz_count += 1;
+        }
+        if (pulses->pulse[n] >= 2 * s_short - s_tolerance
+                && pulses->pulse[n] <= 2 * s_short + s_tolerance) {
+            nrz_width += pulses->pulse[n];
+            nrz_count += 2;
+        }
+        if (pulses->gap[n] >= s_long - s_tolerance
+                && pulses->gap[n] <= s_long + s_tolerance) {
+            nrz_width += pulses->gap[n];
+            nrz_count += 1;
+        }
+        if (pulses->gap[n] >= 2 * s_long - s_tolerance
+                && pulses->gap[n] <= 2 * s_long + s_tolerance) {
+            nrz_width += pulses->gap[n];
+            nrz_count += 2;
+        }
+    }
+    // require at least 10 bits measured
+    if (nrz_count > 20) {
+        f_short = f_long = (float)nrz_count / nrz_width;
+        if (device->verbose > 1) {
+            float to_us = 1e6 / pulses->sample_rate;
+            fprintf(stderr, "%s: Exact bit width (in us) is %.2f vs %.2f, %d bit measured\n", device->name,
+                    to_us / f_short, to_us * s_short, nrz_count);
+        }
+    }
 
     for (unsigned n = 0; n < pulses->num_pulses; ++n) {
         // Determine number of high bit periods for NRZ coding, where bits may not be separated
@@ -357,7 +419,7 @@ int pulse_demod_pwm(const pulse_data_t *pulses, r_device *device)
         }
         else {
             // Pulse outside specified timing
-            return 0;
+            bitbuffer_add_row(&bits);
         }
 
         // End of Message?
@@ -455,6 +517,14 @@ int pulse_demod_manchester_zerobit(const pulse_data_t *pulses, r_device *device)
     return events;
 }
 
+static inline int pulse_demod_get_symbol(const pulse_data_t *pulses, unsigned int n)
+{
+    if (n % 2 == 0)
+        return pulses->pulse[n / 2];
+    else
+        return pulses->gap[n / 2];
+}
+
 int pulse_demod_dmc(const pulse_data_t *pulses, r_device *device)
 {
     float samples_per_us = pulses->sample_rate / 1.0e6;
@@ -477,23 +547,18 @@ int pulse_demod_dmc(const pulse_data_t *pulses, r_device *device)
         return 0;
     }
 
-    int symbol[PD_MAX_PULSES * 2 + 1] = {0};
-    unsigned int n;
-
     bitbuffer_t bits = {0};
     int events = 0;
 
-    for (n = 0; n < pulses->num_pulses; n++) {
-        symbol[n * 2] = pulses->pulse[n];
-        symbol[n * 2 + 1] = pulses->gap[n];
-    }
+    for (unsigned int n = 0; n < pulses->num_pulses * 2; ++n) {
+        int symbol = pulse_demod_get_symbol(pulses, n);
 
-    for (n = 0; n < pulses->num_pulses * 2; ++n) {
-        if (abs(symbol[n] - s_short) < s_tolerance) {
+        if (abs(symbol - s_short) < s_tolerance) {
             // Short - 1
             bitbuffer_add_bit(&bits, 1);
-            if (abs(symbol[++n] - s_short) > s_tolerance) {
-                if (symbol[n] >= s_reset - s_tolerance) {
+            symbol = pulse_demod_get_symbol(pulses, ++n);
+            if (abs(symbol - s_short) > s_tolerance) {
+                if (symbol >= s_reset - s_tolerance) {
                     // Don't expect another short gap at end of message
                     n--;
                 }
@@ -506,11 +571,11 @@ int pulse_demod_dmc(const pulse_data_t *pulses, r_device *device)
                 }
             }
         }
-        else if (abs(symbol[n] - s_long) < s_tolerance) {
+        else if (abs(symbol - s_long) < s_tolerance) {
             // Long - 0
             bitbuffer_add_bit(&bits, 0);
         }
-        else if (symbol[n] >= s_reset - s_tolerance
+        else if (symbol >= s_reset - s_tolerance
                 && bits.num_rows > 0) { // Only if data has been accumulated
             //END message ?
             events += account_event(device, &bits, __func__);
@@ -545,30 +610,23 @@ int pulse_demod_piwm_raw(const pulse_data_t *pulses, r_device *device)
     // precision reciprocal
     float f_short = device->short_width > 0.0 ? 1.0 / (device->short_width * samples_per_us) : 0;
 
-    int symbol[PD_MAX_PULSES * 2];
-    unsigned int n;
     int w;
 
     bitbuffer_t bits = {0};
     int events = 0;
 
-
-    for (n = 0; n < pulses->num_pulses; n++) {
-        symbol[n * 2] = pulses->pulse[n];
-        symbol[n * 2 + 1] = pulses->gap[n];
-    }
-
-    for (n = 0; n < pulses->num_pulses * 2; ++n) {
-        w = symbol[n] * f_short + 0.5;
-        if (symbol[n] > s_long) {
+    for (unsigned int n = 0; n < pulses->num_pulses * 2; ++n) {
+        int symbol = pulse_demod_get_symbol(pulses, n);
+        w = symbol * f_short + 0.5;
+        if (symbol > s_long) {
             bitbuffer_add_row(&bits);
         }
-        else if (abs(symbol[n] - w * s_short) < s_tolerance) {
+        else if (abs(symbol - w * s_short) < s_tolerance) {
             // Add w symbols
             for (; w > 0; --w)
                 bitbuffer_add_bit(&bits, 1 - n % 2);
         }
-        else if (symbol[n] < s_reset
+        else if (symbol < s_reset
                 && bits.num_rows > 0
                 && bits.bits_per_row[bits.num_rows - 1] > 0) {
             bitbuffer_add_row(&bits);
@@ -579,7 +637,7 @@ int pulse_demod_piwm_raw(const pulse_data_t *pulses, r_device *device)
         }
 
         if (((n == pulses->num_pulses * 2 - 1)              // No more pulses? (FSK)
-                    || (symbol[n] > s_reset)) // Long silence (OOK)
+                    || (symbol > s_reset)) // Long silence (OOK)
                 && (bits.num_rows > 0)) {                   // Only if data has been accumulated
             //END message ?
             events += account_event(device, &bits, __func__);
@@ -611,27 +669,20 @@ int pulse_demod_piwm_dc(const pulse_data_t *pulses, r_device *device)
         return 0;
     }
 
-    int symbol[PD_MAX_PULSES * 2];
-    unsigned int n;
-
     bitbuffer_t bits = {0};
     int events = 0;
 
-    for (n = 0; n < pulses->num_pulses; n++) {
-        symbol[n * 2] = pulses->pulse[n];
-        symbol[n * 2 + 1] = pulses->gap[n];
-    }
-
-    for (n = 0; n < pulses->num_pulses * 2; ++n) {
-        if (abs(symbol[n] - s_short) < s_tolerance) {
+    for (unsigned int n = 0; n < pulses->num_pulses * 2; ++n) {
+        int symbol = pulse_demod_get_symbol(pulses, n);
+        if (abs(symbol - s_short) < s_tolerance) {
             // Short - 1
             bitbuffer_add_bit(&bits, 1);
         }
-        else if (abs(symbol[n] - s_long) < s_tolerance) {
+        else if (abs(symbol - s_long) < s_tolerance) {
             // Long - 0
             bitbuffer_add_bit(&bits, 0);
         }
-        else if (symbol[n] < s_reset
+        else if (symbol < s_reset
                 && bits.num_rows > 0
                 && bits.bits_per_row[bits.num_rows - 1] > 0) {
             bitbuffer_add_row(&bits);
@@ -642,7 +693,7 @@ int pulse_demod_piwm_dc(const pulse_data_t *pulses, r_device *device)
         }
 
         if (((n == pulses->num_pulses * 2 - 1)              // No more pulses? (FSK)
-                    || (symbol[n] > s_reset)) // Long silence (OOK)
+                    || (symbol > s_reset)) // Long silence (OOK)
                 && (bits.num_rows > 0)) {                   // Only if data has been accumulated
             //END message ?
             events += account_event(device, &bits, __func__);
