@@ -20,13 +20,17 @@
 #include "r_util.h"
 #include "rtl_433.h"
 #include "r_private.h"
+#include "rtl_433_devices.h"
 #include "r_device.h"
 #include "pulse_demod.h"
 #include "pulse_detect_fsk.h"
 #include "sdr.h"
 #include "data.h"
+#include "data_tag.h"
 #include "list.h"
 #include "optparse.h"
+#include "output_file.h"
+#include "output_udp.h"
 #include "output_mqtt.h"
 #include "output_influx.h"
 #include "write_sigrok.h"
@@ -59,8 +63,12 @@ char const *version_string(void)
 #define STR_VALUE(arg) #arg
 #define STR_EXPAND(s) STR_VALUE(s)
             " version " STR_EXPAND(GIT_VERSION)
+#ifdef GIT_BRANCH
             " branch " STR_EXPAND(GIT_BRANCH)
+#endif
+#ifdef GIT_TIMESTAMP
             " at " STR_EXPAND(GIT_TIMESTAMP)
+#endif
 #undef STR_VALUE
 #undef STR_EXPAND
 #else
@@ -72,6 +80,9 @@ char const *version_string(void)
 #endif
 #ifdef SOAPYSDR
             " SoapySDR"
+#endif
+#ifdef OPENSSL
+            " with TLS"
 #endif
             ;
 }
@@ -95,18 +106,19 @@ void set_center_freq(r_cfg_t *cfg, uint32_t center_freq)
     cfg->frequencies = 1;
     cfg->frequency_index = 0;
     cfg->frequency[0] = center_freq;
+    // cfg->center_frequency = center_freq; // actually applied in the sdr event
     sdr_set_center_freq(cfg->dev, center_freq, 0);
 }
 
 void set_freq_correction(r_cfg_t *cfg, int freq_correction)
 {
-    cfg->ppm_error = freq_correction;
+    // cfg->ppm_error = freq_correction; // actually applied in the sdr event
     sdr_set_freq_correction(cfg->dev, freq_correction, 0);
 }
 
 void set_sample_rate(r_cfg_t *cfg, uint32_t sample_rate)
 {
-    cfg->samp_rate = sample_rate;
+    // cfg->samp_rate = sample_rate; // actually applied in the sdr event
     sdr_set_sample_rate(cfg->dev, sample_rate, 0);
 }
 
@@ -136,6 +148,23 @@ void r_init_cfg(r_cfg_t *cfg)
     list_ensure_size(&cfg->in_files, 100);
     list_ensure_size(&cfg->output_handler, 16);
 
+    // collect devices list, this should be a module
+    r_device r_devices[] = {
+#define DECL(name) name,
+            DEVICES
+#undef DECL
+    };
+
+    cfg->num_r_devices = sizeof(r_devices) / sizeof(*r_devices);
+    for (unsigned i = 0; i < cfg->num_r_devices; i++) {
+        r_devices[i].protocol_num = i + 1;
+    }
+    cfg->devices = malloc(sizeof(r_devices));
+    if (!cfg->devices)
+        FATAL_CALLOC("r_init_cfg()");
+
+    memcpy(cfg->devices, r_devices, sizeof(r_devices));
+
     cfg->demod = calloc(1, sizeof(*cfg->demod));
     if (!cfg->demod)
         FATAL_CALLOC("r_init_cfg()");
@@ -144,6 +173,11 @@ void r_init_cfg(r_cfg_t *cfg)
     cfg->demod->min_level = -12.1442;
     cfg->demod->min_snr = 9.0;
 
+    // note: this should be optional
+    cfg->demod->pulse_detect = pulse_detect_create();
+    // initialize tables
+    baseband_init();
+
     time(&cfg->frames_since);
 
     list_ensure_size(&cfg->demod->r_devs, 100);
@@ -177,7 +211,7 @@ void r_free_cfg(r_cfg_t *cfg)
     }
     list_free_elems(&cfg->demod->dumper, free);
 
-    list_free_elems(&cfg->demod->r_devs, free);
+    list_free_elems(&cfg->demod->r_devs, (list_elem_free_fn)free_protocol);
 
     if (cfg->demod->am_analyze)
         am_analyze_free(cfg->demod->am_analyze);
@@ -186,8 +220,12 @@ void r_free_cfg(r_cfg_t *cfg)
 
     free(cfg->demod);
 
+    free(cfg->devices);
+
     list_free_elems(&cfg->output_handler, (list_elem_free_fn)data_output_free);
 
+    list_free_elems(&cfg->data_tags, (list_elem_free_fn)data_tag_free);
+
     list_free_elems(&cfg->in_files, NULL);
 
     mg_mgr_free(cfg->mgr);
@@ -200,6 +238,18 @@ void r_free_cfg(r_cfg_t *cfg)
 
 void register_protocol(r_cfg_t *cfg, r_device *r_dev, char *arg)
 {
+    // use arg of 'v', 'vv', 'vvv' as device verbosity
+    int dev_verbose = 0;
+    if (arg && *arg == 'v') {
+        for (; *arg == 'v'; ++arg) {
+            dev_verbose++;
+        }
+        if (*arg) {
+            arg++; // skip separator
+        }
+    }
+
+    // use any other arg as device parameter
     r_device *p;
     if (r_dev->create_fn) {
         p = r_dev->create_fn(arg);
@@ -214,11 +264,9 @@ void register_protocol(r_cfg_t *cfg, r_device *r_dev, char *arg)
         *p = *r_dev; // copy
     }
 
-    p->verbose      = cfg->verbosity > 0 ? cfg->verbosity - 1 : 0;
+    p->verbose      = dev_verbose ? dev_verbose : (cfg->verbosity > 0 ? cfg->verbosity - 1 : 0);
     p->verbose_bits = cfg->verbose_bits;
 
-    p->old_model_keys = cfg->old_model_keys; // TODO: temporary allow to change to new style model keys
-
     p->output_fn  = data_acquired_handler;
     p->output_ctx = cfg;
 
@@ -268,16 +316,23 @@ void calc_rssi_snr(r_cfg_t *cfg, pulse_data_t *pulse_data)
     float foffs2 = (float)pulse_data->fsk_f2_est / INT16_MAX * cfg->samp_rate / 2.0;
     pulse_data->freq1_hz = (foffs1 + cfg->center_frequency);
     pulse_data->freq2_hz = (foffs2 + cfg->center_frequency);
+    pulse_data->centerfreq_hz = cfg->center_frequency;
+    pulse_data->depth_bits    = cfg->demod->sample_size * 4;
     // NOTE: for (CU8) amplitude is 10x (because it's squares)
-    if (cfg->demod->sample_size == 1 && !cfg->demod->use_mag_est) { // amplitude (CU8)
-        pulse_data->rssi_db = 10.0f * log10f(ook_high_estimate) - 42.1442f; // 10*log10f(16384.0f)
+    if (cfg->demod->sample_size == 2 && !cfg->demod->use_mag_est) { // amplitude (CU8)
+        pulse_data->range_db = 42.1442f; // 10*log10f(16384.0f) == 20*log10f(128.0f)
+        pulse_data->rssi_db  = 10.0f * log10f(ook_high_estimate) - 42.1442f; // 10*log10f(16384.0f)
         pulse_data->noise_db = 10.0f * log10f(ook_low_estimate) - 42.1442f; // 10*log10f(16384.0f)
-        pulse_data->snr_db  = 10.0f * log10f(asnr);
+        pulse_data->snr_db   = 10.0f * log10f(asnr);
     }
-    else { // magnitude (CS16)
-        pulse_data->rssi_db = 20.0f * log10f(ook_high_estimate) - 84.2884f; // 20*log10f(16384.0f)
+    else { // magnitude (CU8, CS16)
+        pulse_data->range_db = 84.2884f; // 20*log10f(16384.0f)
+        // lowest (scaled x128) reading at  8 bit is -20*log10(128) = -42.1442 (eff. -36 dB)
+        // lowest (scaled div2) reading at 12 bit is -20*log10(1024) = -60.2060 (eff. -54 dB)
+        // lowest (scaled div2) reading at 16 bit is -20*log10(16384) = -84.2884 (eff. -78 dB)
+        pulse_data->rssi_db  = 20.0f * log10f(ook_high_estimate) - 84.2884f; // 20*log10f(16384.0f)
         pulse_data->noise_db = 20.0f * log10f(ook_low_estimate) - 84.2884f; // 20*log10f(16384.0f)
-        pulse_data->snr_db  = 20.0f * log10f(asnr);
+        pulse_data->snr_db   = 20.0f * log10f(asnr);
     }
 }
 
@@ -316,44 +371,48 @@ char *time_pos_str(r_cfg_t *cfg, unsigned samples_ago, char *buf)
 // well-known field "protocol" is only used when model protocol is requested
 // well-known field "description" is only used when model description is requested
 // well-known fields "mod", "freq", "freq1", "freq2", "rssi", "snr", "noise" are used by meta report option
-static char const *well_known_default[15] = {0};
 char const **well_known_output_fields(r_cfg_t *cfg)
 {
-    char const **p = well_known_default;
-    *p++ = "time";
-    *p++ = "msg";
-    *p++ = "codes";
+    list_t field_list = {0};
+    list_ensure_size(&field_list, 15);
+
+    list_push(&field_list, "time");
+    list_push(&field_list, "msg");
+    list_push(&field_list, "codes");
 
     if (cfg->verbose_bits)
-        *p++ = "bits";
-    if (cfg->output_key)
-        *p++ = cfg->output_key;
-    if (cfg->report_protocol)
-        *p++ = "protocol";
-    if (cfg->report_description)
-        *p++ = "description";
-    if (cfg->report_meta) {
-        *p++ = "mod";
-        *p++ = "freq";
-        *p++ = "freq1";
-        *p++ = "freq2";
-        *p++ = "rssi";
-        *p++ = "snr";
-        *p++ = "noise";
+        list_push(&field_list, "bits");
+
+    for (void **iter = cfg->data_tags.elems; iter && *iter; ++iter) {
+        data_tag_t *tag = *iter;
+        if (tag->key) {
+            list_push(&field_list, (void *)tag->key);
+        }
+        else {
+            list_push_all(&field_list, (void **)tag->includes);
+        }
     }
 
-    return well_known_default;
+    if (cfg->report_protocol)
+        list_push(&field_list, "protocol");
+    if (cfg->report_description)
+        list_push(&field_list, "description");
+    if (cfg->report_meta) {
+        list_push(&field_list, "mod");
+        list_push(&field_list, "freq");
+        list_push(&field_list, "freq1");
+        list_push(&field_list, "freq2");
+        list_push(&field_list, "rssi");
+        list_push(&field_list, "snr");
+        list_push(&field_list, "noise");
+    }
+
+    return (char const **)field_list.elems;
 }
 
 /** Convert CSV keys according to selected conversion mode. Replacement is static but in-place. */
 static char const **convert_csv_fields(r_cfg_t *cfg, char const **fields)
 {
-    if (!cfg->old_model_keys) {
-        for (char const **p = fields; *p; ++p) {
-            if (!strcmp(*p, "battery")) *p = "battery_ok";
-        }
-    }
-
     if (cfg->conversion_mode == CONVERT_SI) {
         for (char const **p = fields; *p; ++p) {
             if (!strcmp(*p, "temperature_F")) *p = "temperature_C";
@@ -383,7 +442,7 @@ static char const **convert_csv_fields(r_cfg_t *cfg, char const **fields)
 }
 
 // find the fields output for CSV
-char const **determine_csv_fields(r_cfg_t *cfg, char const **well_known, int *num_fields)
+char const **determine_csv_fields(r_cfg_t *cfg, char const *const *well_known, int *num_fields)
 {
     list_t field_list = {0};
     list_ensure_size(&field_list, 100);
@@ -411,43 +470,56 @@ int run_ook_demods(list_t *r_devs, pulse_data_t *pulse_data)
 {
     int p_events = 0;
 
-    for (void **iter = r_devs->elems; iter && *iter; ++iter) {
-        r_device *r_dev = *iter;
-        switch (r_dev->modulation) {
-        case OOK_PULSE_PCM_RZ:
-            p_events += pulse_demod_pcm(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_PPM:
-            p_events += pulse_demod_ppm(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_PWM:
-            p_events += pulse_demod_pwm(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_MANCHESTER_ZEROBIT:
-            p_events += pulse_demod_manchester_zerobit(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_PIWM_RAW:
-            p_events += pulse_demod_piwm_raw(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_PIWM_DC:
-            p_events += pulse_demod_piwm_dc(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_DMC:
-            p_events += pulse_demod_dmc(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_PWM_OSV1:
-            p_events += pulse_demod_osv1(pulse_data, r_dev);
-            break;
-        case OOK_PULSE_NRZS:
-            p_events += pulse_demod_nrzs(pulse_data, r_dev);
-            break;
-        // FSK decoders
-        case FSK_PULSE_PCM:
-        case FSK_PULSE_PWM:
-        case FSK_PULSE_MANCHESTER_ZEROBIT:
-            break;
-        default:
-            fprintf(stderr, "Unknown modulation %u in protocol!\n", r_dev->modulation);
+    unsigned next_priority = 0; // next smallest on each loop through decoders
+    // run all decoders of each priority, stop if an event is produced
+    for (unsigned priority = 0; !p_events && priority < UINT_MAX; priority = next_priority) {
+        next_priority = UINT_MAX;
+        for (void **iter = r_devs->elems; iter && *iter; ++iter) {
+            r_device *r_dev = *iter;
+
+            // Find next smallest priority
+            if (r_dev->priority > priority && r_dev->priority < next_priority)
+                next_priority = r_dev->priority;
+            // Run only current priority
+            if (r_dev->priority != priority)
+                continue;
+
+            switch (r_dev->modulation) {
+            case OOK_PULSE_PCM_RZ:
+                p_events += pulse_demod_pcm(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_PPM:
+                p_events += pulse_demod_ppm(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_PWM:
+                p_events += pulse_demod_pwm(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_MANCHESTER_ZEROBIT:
+                p_events += pulse_demod_manchester_zerobit(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_PIWM_RAW:
+                p_events += pulse_demod_piwm_raw(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_PIWM_DC:
+                p_events += pulse_demod_piwm_dc(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_DMC:
+                p_events += pulse_demod_dmc(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_PWM_OSV1:
+                p_events += pulse_demod_osv1(pulse_data, r_dev);
+                break;
+            case OOK_PULSE_NRZS:
+                p_events += pulse_demod_nrzs(pulse_data, r_dev);
+                break;
+            // FSK decoders
+            case FSK_PULSE_PCM:
+            case FSK_PULSE_PWM:
+            case FSK_PULSE_MANCHESTER_ZEROBIT:
+                break;
+            default:
+                fprintf(stderr, "Unknown modulation %u in protocol!\n", r_dev->modulation);
+            }
         }
     }
 
@@ -458,31 +530,44 @@ int run_fsk_demods(list_t *r_devs, pulse_data_t *fsk_pulse_data)
 {
     int p_events = 0;
 
-    for (void **iter = r_devs->elems; iter && *iter; ++iter) {
-        r_device *r_dev = *iter;
-        switch (r_dev->modulation) {
-        // OOK decoders
-        case OOK_PULSE_PCM_RZ:
-        case OOK_PULSE_PPM:
-        case OOK_PULSE_PWM:
-        case OOK_PULSE_MANCHESTER_ZEROBIT:
-        case OOK_PULSE_PIWM_RAW:
-        case OOK_PULSE_PIWM_DC:
-        case OOK_PULSE_DMC:
-        case OOK_PULSE_PWM_OSV1:
-        case OOK_PULSE_NRZS:
-            break;
-        case FSK_PULSE_PCM:
-            p_events += pulse_demod_pcm(fsk_pulse_data, r_dev);
-            break;
-        case FSK_PULSE_PWM:
-            p_events += pulse_demod_pwm(fsk_pulse_data, r_dev);
-            break;
-        case FSK_PULSE_MANCHESTER_ZEROBIT:
-            p_events += pulse_demod_manchester_zerobit(fsk_pulse_data, r_dev);
-            break;
-        default:
-            fprintf(stderr, "Unknown modulation %u in protocol!\n", r_dev->modulation);
+    unsigned next_priority = 0; // next smallest on each loop through decoders
+    // run all decoders of each priority, stop if an event is produced
+    for (unsigned priority = 0; !p_events && priority < UINT_MAX; priority = next_priority) {
+        next_priority = UINT_MAX;
+        for (void **iter = r_devs->elems; iter && *iter; ++iter) {
+            r_device *r_dev = *iter;
+
+            // Find next smallest priority
+            if (r_dev->priority > priority && r_dev->priority < next_priority)
+                next_priority = r_dev->priority;
+            // Run only current priority
+            if (r_dev->priority != priority)
+                continue;
+
+            switch (r_dev->modulation) {
+            // OOK decoders
+            case OOK_PULSE_PCM_RZ:
+            case OOK_PULSE_PPM:
+            case OOK_PULSE_PWM:
+            case OOK_PULSE_MANCHESTER_ZEROBIT:
+            case OOK_PULSE_PIWM_RAW:
+            case OOK_PULSE_PIWM_DC:
+            case OOK_PULSE_DMC:
+            case OOK_PULSE_PWM_OSV1:
+            case OOK_PULSE_NRZS:
+                break;
+            case FSK_PULSE_PCM:
+                p_events += pulse_demod_pcm(fsk_pulse_data, r_dev);
+                break;
+            case FSK_PULSE_PWM:
+                p_events += pulse_demod_pwm(fsk_pulse_data, r_dev);
+                break;
+            case FSK_PULSE_MANCHESTER_ZEROBIT:
+                p_events += pulse_demod_manchester_zerobit(fsk_pulse_data, r_dev);
+                break;
+            default:
+                fprintf(stderr, "Unknown modulation %u in protocol!\n", r_dev->modulation);
+            }
         }
     }
 
@@ -530,23 +615,6 @@ void data_acquired_handler(r_device *r_dev, data_t *data)
     }
 #endif
 
-    // replace textual battery key with numerical battery key
-    if (!cfg->old_model_keys) {
-        for (data_t *d = data; d; d = d->next) {
-            if ((d->type == DATA_STRING) && !strcmp(d->key, "battery")) {
-                free(d->key);
-                d->key = strdup("battery_ok");
-                if (!d->key)
-                    FATAL_STRDUP("data_acquired_handler()");
-                int ok = d->value.v_ptr && !strcmp(d->value.v_ptr, "OK");
-                free(d->value.v_ptr);
-                d->type = DATA_INT;
-                d->value.v_int = ok;
-                break;
-            }
-        }
-    }
-
     if (cfg->conversion_mode == CONVERT_SI) {
         for (data_t *d = data; d; d = d->next) {
             // Convert double type fields ending in _F to _C
@@ -742,18 +810,10 @@ void data_acquired_handler(r_device *r_dev, data_t *data)
                 NULL);
     }
 
-    // prepend "tag" if available
-    if (cfg->output_tag) {
-        char const *output_tag = cfg->output_tag;
-        if (cfg->in_filename && !strcmp("PATH", cfg->output_tag)) {
-            output_tag = cfg->in_filename;
-        }
-        else if (cfg->in_filename && !strcmp("FILE", cfg->output_tag)) {
-            output_tag = file_basename(cfg->in_filename);
-        }
-        data = data_prepend(data,
-                cfg->output_key, "", DATA_STRING, output_tag,
-                NULL);
+    // apply all tags
+    for (void **iter = cfg->data_tags.elems; iter && *iter; ++iter) {
+        data_tag_t *tag = *iter;
+        data            = data_tag_apply(tag, data, cfg->in_filename);
     }
 
     for (size_t i = 0; i < cfg->output_handler.len; ++i) { // list might contain NULLs
@@ -880,10 +940,10 @@ void add_csv_output(r_cfg_t *cfg, char *param)
     list_push(&cfg->output_handler, data_output_csv_create(fopen_output(param)));
 }
 
-void start_outputs(r_cfg_t *cfg, char const **well_known)
+void start_outputs(r_cfg_t *cfg, char const *const *well_known)
 {
     int num_output_fields;
-    const char **output_fields = determine_csv_fields(cfg, well_known, &num_output_fields);
+    char const **output_fields = determine_csv_fields(cfg, well_known, &num_output_fields);
 
     for (size_t i = 0; i < cfg->output_handler.len; ++i) { // list might contain NULLs
         data_output_start(cfg->output_handler.elems[i], output_fields, num_output_fields);
@@ -899,12 +959,7 @@ void add_kv_output(r_cfg_t *cfg, char *param)
 
 void add_mqtt_output(r_cfg_t *cfg, char *param)
 {
-    char *host = "localhost";
-    char *port = "1883";
-    char *opts = hostport_param(param, &host, &port);
-    fprintf(stderr, "Publishing MQTT data to %s port %s\n", host, port);
-
-    list_push(&cfg->output_handler, data_output_mqtt_create(get_mgr(cfg), host, port, opts, cfg->dev_query));
+    list_push(&cfg->output_handler, data_output_mqtt_create(get_mgr(cfg), param, cfg->dev_query));
 }
 
 void add_influx_output(r_cfg_t *cfg, char *param)
@@ -990,7 +1045,7 @@ void add_dumper(r_cfg_t *cfg, char const *spec, int overwrite)
         FATAL_CALLOC("add_dumper()");
     list_push(&cfg->demod->dumper, dumper);
 
-    parse_file_info(spec, dumper);
+    file_info_parse_filename(dumper, spec);
     if (strcmp(dumper->path, "-") == 0) { /* Write samples to stdout */
         dumper->file = stdout;
 #ifdef _WIN32
@@ -1020,3 +1075,8 @@ void add_infile(r_cfg_t *cfg, char *in_file)
 {
     list_push(&cfg->in_files, in_file);
 }
+
+void add_data_tag(struct r_cfg *cfg, char *param)
+{
+    list_push(&cfg->data_tags, data_tag_create(param, get_mgr(cfg)));
+}
