@@ -6,9 +6,9 @@
     Based on rtl_sdr
     Copyright (C) 2012 by Steve Markgraf <steve@steve-m.de>
 
-    This program is free software: you can redistribute it and/or modify
+    This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
+    the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -31,7 +31,6 @@
 #include "rtl_433.h"
 #include "r_private.h"
 #include "r_device.h"
-#include "rtl_433_devices.h"
 #include "r_api.h"
 #include "sdr.h"
 #include "baseband.h"
@@ -43,11 +42,13 @@
 #include "data.h"
 #include "r_util.h"
 #include "optparse.h"
+#include "abuf.h"
 #include "fileformat.h"
 #include "samp_grab.h"
 #include "am_analyze.h"
 #include "confparse.h"
 #include "term_ctl.h"
+#include "compat_alarm.h"
 #include "compat_paths.h"
 #include "fatal.h"
 #include "write_sigrok.h"
@@ -115,9 +116,13 @@ static void usage(int exit_code)
             "       Specify a negative number to disable a device decoding protocol (can be used multiple times)\n"
             "  [-G] Enable blacklisted device decoding protocols, for testing only.\n"
             "  [-X <spec> | help] Add a general purpose decoder (prepend -R 0 to disable all decoders)\n"
-            "  [-Y level=<dB level>] Manual detection level used to determine pulses (-1.0 to -30.0) (0=auto)\n"
-            "  [-n <value>] Specify number of samples to take (each sample is 2 bytes: 1 each of I & Q)\n"
             "  [-Y auto | classic | minmax] FSK pulse detector mode.\n"
+            "  [-Y level=<dB level>] Manual detection level used to determine pulses (-1.0 to -30.0) (0=auto).\n"
+            "  [-Y minlevel=<dB level>] Manual minimum detection level used to determine pulses (-1.0 to -99.0).\n"
+            "  [-Y minsnr=<dB level>] Minimum SNR to determine pulses (1.0 to 99.0).\n"
+            "  [-Y autolevel] Set minlevel automatically based on average estimated noise.\n"
+            "  [-Y squelch] Skip frames below estimated noise level to reduce cpu load.\n"
+            "  [-Y ampest | magest] Choose amplitude or magnitude level estimator.\n"
             "\t\t= Analyze/Debug options =\n"
             "  [-a] Analyze mode. Print a textual description of the signal.\n"
             "  [-A] Pulse Analyzer. Enable pulse analysis and decode attempt.\n"
@@ -133,9 +138,10 @@ static void usage(int exit_code)
             "  [-F kv | json | csv | mqtt | influx | syslog | null | help] Produce decoded output in given format.\n"
             "       Append output to file with :<filename> (e.g. -F csv:log.csv), defaults to stdout.\n"
             "       Specify host/port for syslog with e.g. -F syslog:127.0.0.1:1514\n"
-            "  [-M time[:<options>] | protocol | level | stats | bits | help] Add various meta data to each output.\n"
-            "  [-K FILE | PATH | <tag> | <key>=<value>] Add an expanded token or fixed tag to every output line.\n"
+            "  [-M time[:<options>] | protocol | level | noise[:secs] | stats | bits | help] Add various meta data to each output.\n"
+            "  [-K FILE | PATH | <tag> | <key>=<tag>] Add an expanded token or fixed tag to every output line.\n"
             "  [-C native | si | customary] Convert units in decoded output.\n"
+            "  [-n <value>] Specify number of samples to take (each sample is an I/Q pair)\n"
             "  [-T <seconds>] Specify number of seconds to run, also 12:34 or 1h23m45s\n"
             "  [-E hop | quit] Hop/Quit after outputting successful event(s)\n"
             "  [-h] Output this usage help and exit\n"
@@ -210,7 +216,7 @@ static void help_output(void)
             "\tAppend output to file with :<filename> (e.g. -F csv:log.csv), defaults to stdout.\n"
             "\tSpecify MQTT server with e.g. -F mqtt://localhost:1883\n"
             "\tAdd MQTT options with e.g. -F \"mqtt://host:1883,opt=arg\"\n"
-            "\tMQTT options are: user=foo, pass=bar, retain[=0|1], <format>[=topic]\n"
+            "\tMQTT options are: user=foo, pass=bar, retain[=0|1], qos=N, <format>[=topic]\n"
             "\tSupported MQTT formats: (default is all)\n"
             "\t  events: posts JSON event data\n"
             "\t  states: posts JSON state data\n"
@@ -226,12 +232,33 @@ static void help_output(void)
     exit(0);
 }
 
+_Noreturn
+static void help_tags(void)
+{
+    term_help_printf(
+            "\t\t= Data tags option =\n"
+            "  [-K FILE | PATH | <tag> | <key>=<tag>] Add an expanded token or fixed tag to every output line.\n"
+            "\tIf <tag> is \"FILE\" or \"PATH\" an expanded token will be added.\n"
+            "\tThe <tag> can also be a GPSd URL, e.g.\n"
+            "\t\t\"-K gpsd,lat,lon\" (report lat and lon keys from local gpsd)\n"
+            "\t\t\"-K loc=gpsd,lat,lon\" (report lat and lon in loc object)\n"
+            "\t\t\"-K gpsd\" (full json TPV report, in default \"gps\" object)\n"
+            "\t\t\"-K foo=gpsd://127.0.0.1:2947\" (with key and address)\n"
+            "\t\t\"-K bar=gpsd,nmea\" (NMEA deault GPGGA report)\n"
+            "\t\t\"-K rmc=gpsd,nmea,filter='$GPRMC'\" (NMEA GPRMC report)\n"
+            "\tAlso <tag> can be a generic tcp address, e.g.\n"
+            "\t\t\"-K foo=tcp:localhost:4000\" (read lines as TCP client)\n"
+            "\t\t\"-K bar=tcp://127.0.0.1:3000,init='subscribe tags\\r\\n'\"\n"
+            "\t\t\"-K baz=tcp://127.0.0.1:5000,filter='a prefix to match'\"\n");
+    exit(0);
+}
+
 _Noreturn
 static void help_meta(void)
 {
     term_help_printf(
             "\t\t= Meta information option =\n"
-            "  [-M time[:<options>]|protocol|level|stats|bits|oldmodel] Add various metadata to every output line.\n"
+            "  [-M time[:<options>]|protocol|level|noise[:<secs>]|stats|bits] Add various metadata to every output line.\n"
             "\tUse \"time\" to add current date and time meta data (preset for live inputs).\n"
             "\tUse \"time:rel\" to add sample position meta data (preset for read-file and stdin).\n"
             "\tUse \"time:unix\" to show the seconds since unix epoch as time meta data.\n"
@@ -244,10 +271,10 @@ static void help_meta(void)
             "\t\t\"usec\" and \"utc\" can be combined with other options, eg. \"time:unix:utc:usec\".\n"
             "\tUse \"protocol\" / \"noprotocol\" to output the decoder protocol number meta data.\n"
             "\tUse \"level\" to add Modulation, Frequency, RSSI, SNR, and Noise meta data.\n"
+            "\tUse \"noise[:secs]\" to report estimated noise level at intervals (default: 10 seconds).\n"
             "\tUse \"stats[:[<level>][:<interval>]]\" to report statistics (default: 600 seconds).\n"
             "\t  level 0: no report, 1: report successful devices, 2: report active devices, 3: report all\n"
-            "\tUse \"bits\" to add bit representation to code outputs (for debug).\n"
-            "\tNote: You can use \"oldmodel\" to get the old model keys. This will be removed shortly.\n");
+            "\tUse \"bits\" to add bit representation to code outputs (for debug).\n");
     exit(0);
 }
 
@@ -282,7 +309,7 @@ static void help_write(void)
             "  [-W <filename>] Save data stream to output file, overwrite existing file\n"
             "\tParameters are detected from the full path, file name, and extension.\n\n"
             "\tFile content and format are detected as parameters, possible options are:\n"
-            "\t'cu8', 'cs16', 'cf32' ('IQ' implied),\n"
+            "\t'cu8', 'cs8', 'cs16', 'cf32' ('IQ' implied),\n"
             "\t'am.s16', 'am.f32', 'fm.s16', 'fm.f32',\n"
             "\t'i.f32', 'q.f32', 'logic.u8', 'ook', and 'vcd'.\n\n"
             "\tParameters must be separated by non-alphanumeric chars and are case-insensitive.\n"
@@ -304,10 +331,12 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
         cfg->exit_async = 1;
     }
 
+    // save last frame time to see if a new second started
+    time_t last_frame_sec = demod->now.tv_sec;
     get_time_now(&demod->now);
 
-    n_samples = len / 2 / demod->sample_size;
-    if (n_samples * 2 * demod->sample_size != len) {
+    n_samples = len / demod->sample_size;
+    if (n_samples * demod->sample_size != len) {
         fprintf(stderr, "Sample buffer length not aligned to sample size!\n");
     }
     if (!n_samples) {
@@ -321,31 +350,60 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
     if (demod->frame_end_ago)
         demod->frame_end_ago += n_samples;
 
-#ifndef _WIN32
     alarm(3); // require callback to run every 3 second, abort otherwise
-#endif
 
     if (demod->samp_grab) {
         samp_grab_push(demod->samp_grab, iq_buf, len);
     }
 
     // AM demodulation
-    if (demod->sample_size == 1) { // CU8
+    float avg_db;
+    if (demod->sample_size == 2) { // CU8
         if (demod->use_mag_est) {
             //magnitude_true_cu8(iq_buf, demod->buf.temp, n_samples);
-            magnitude_est_cu8(iq_buf, demod->buf.temp, n_samples);
+            avg_db = magnitude_est_cu8(iq_buf, demod->buf.temp, n_samples);
         }
         else { // amp est
-            envelope_detect(iq_buf, demod->buf.temp, n_samples);
+            avg_db = envelope_detect(iq_buf, demod->buf.temp, n_samples);
         }
     } else { // CS16
         //magnitude_true_cs16((int16_t *)iq_buf, demod->buf.temp, n_samples);
-        magnitude_est_cs16((int16_t *)iq_buf, demod->buf.temp, n_samples);
+        avg_db = magnitude_est_cs16((int16_t *)iq_buf, demod->buf.temp, n_samples);
     }
+
+    //fprintf(stderr, "noise level: %.1f dB current: %.1f dB min level: %.1f dB\n", demod->noise_level, avg_db, demod->min_level_auto);
+    if (demod->min_level_auto == 0.0f) {
+        demod->min_level_auto = demod->min_level;
+    }
+    if (demod->noise_level == 0.0f) {
+        demod->noise_level = demod->min_level_auto - 3.0f;
+    }
+    int noise_only = avg_db < demod->noise_level + 3.0f; // or demod->min_level_auto?
+    // always process frames if loader, dumper, or analyzers are in use, otherwise skip silent frames
+    int process_frame = demod->squelch_offset <= 0 || !noise_only || demod->load_info.format || demod->analyze_pulses || demod->dumper.len || demod->samp_grab;
+    if (noise_only) {
+        demod->noise_level = (demod->noise_level * 7 + avg_db) / 8; // fast fall over 8 frames
+        // If auto_level and noise level well below min_level and significant change in noise level
+        if (demod->auto_level > 0 && demod->noise_level < demod->min_level - 3.0f
+                && fabsf(demod->min_level_auto - demod->noise_level - 3.0f) > 1.0f) {
+            demod->min_level_auto = demod->noise_level + 3.0f;
+            fprintf(stderr, "Estimated noise level is %.1f dB, adjusting minimum detection level to %.1f dB\n", demod->noise_level, demod->min_level_auto);
+            pulse_detect_set_levels(demod->pulse_detect, demod->use_mag_est, demod->level_limit, demod->min_level_auto, demod->min_snr, demod->detect_verbosity);
+        }
+    } else {
+        demod->noise_level = (demod->noise_level * 31 + avg_db) / 32; // slow rise over 32 frames
+    }
+    // Report noise every report_noise seconds, but only for the first frame that second
+    if (cfg->report_noise && last_frame_sec != demod->now.tv_sec && demod->now.tv_sec % cfg->report_noise == 0) {
+        fprintf(stderr, "Current %s level %.1f dB, estimated noise %.1f dB\n",
+                noise_only ? "noise" : "signal", avg_db, demod->noise_level);
+    }
+
+    if (process_frame)
     baseband_low_pass_filter(demod->buf.temp, demod->am_buf, n_samples, &demod->lowpass_filter_state);
 
     // FM demodulation
-    /* Select the correct fsk pulse detector */
+    // Select the correct fsk pulse detector
     unsigned fpdm = cfg->fsk_pulse_detect_mode;
     if (cfg->fsk_pulse_detect_mode == FSK_PULSE_DETECT_AUTO) {
         if (cfg->frequency[cfg->frequency_index] > FSK_PULSE_DETECTOR_LIMIT)
@@ -354,19 +412,24 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
             fpdm = FSK_PULSE_DETECT_OLD;
     }
 
-    if (demod->enable_FM_demod) {
-        if (demod->sample_size == 1) { // CU8
-            baseband_demod_FM(iq_buf, demod->buf.fm, n_samples, &demod->demod_FM_state, fpdm);
+    if (demod->enable_FM_demod && process_frame) {
+        float low_pass = demod->low_pass != 0.0f ? demod->low_pass : fpdm ? 0.2f : 0.1f;
+        if (demod->sample_size == 2) { // CU8
+            baseband_demod_FM(iq_buf, demod->buf.fm, n_samples, cfg->samp_rate, low_pass, &demod->demod_FM_state);
         } else { // CS16
-            baseband_demod_FM_cs16((int16_t *)iq_buf, demod->buf.fm, n_samples, &demod->demod_FM_state, fpdm);
+            baseband_demod_FM_cs16((int16_t *)iq_buf, demod->buf.fm, n_samples, cfg->samp_rate, low_pass, &demod->demod_FM_state);
         }
     }
 
     // Handle special input formats
     if (demod->load_info.format == S16_AM) { // The IQ buffer is really AM demodulated data
+        if (len > sizeof(demod->am_buf))
+            FATAL("Buffer too small");
         memcpy(demod->am_buf, iq_buf, len);
     } else if (demod->load_info.format == S16_FM) { // The IQ buffer is really FM demodulated data
         // we would need AM for the envelope too
+        if (len > sizeof(demod->buf.fm))
+            FATAL("Buffer too small");
         memcpy(demod->buf.fm, iq_buf, len);
     }
 
@@ -381,7 +444,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
                 break;
             }
         }
-        while (package_type) {
+        while (package_type && process_frame) {
             int p_events = 0; // Sensor events successfully detected per package
             package_type = pulse_detect_package(demod->pulse_detect, demod->am_buf, demod->buf.fm, n_samples, cfg->samp_rate, cfg->input_pos, &demod->pulse_data, &demod->fsk_pulse_data, fpdm);
             if (package_type) {
@@ -407,6 +470,10 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
                 }
 
                 if (cfg->verbosity > 2) pulse_data_print(&demod->pulse_data);
+                if (cfg->raw_mode == 1 || (cfg->raw_mode == 2 && p_events == 0) || (cfg->raw_mode == 3 && p_events > 0)) {
+                    data_t *data = pulse_data_print_data(&demod->pulse_data);
+                    event_occurred_handler(cfg, data);
+                }
                 if (demod->analyze_pulses && (cfg->grab_mode <= 1 || (cfg->grab_mode == 2 && p_events == 0) || (cfg->grab_mode == 3 && p_events > 0)) ) {
                     pulse_analyzer(&demod->pulse_data, package_type);
                 }
@@ -427,7 +494,11 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
                 }
 
                 if (cfg->verbosity > 2) pulse_data_print(&demod->fsk_pulse_data);
-                if (demod->analyze_pulses && (cfg->grab_mode <= 1 || (cfg->grab_mode == 2 && p_events == 0) || (cfg->grab_mode == 3 && p_events > 0)) ) {
+                if (cfg->raw_mode == 1 || (cfg->raw_mode == 2 && p_events == 0) || (cfg->raw_mode == 3 && p_events > 0)) {
+                    data_t *data = pulse_data_print_data(&demod->fsk_pulse_data);
+                    event_occurred_handler(cfg, data);
+                }
+                if (demod->analyze_pulses && (cfg->grab_mode <= 1 || (cfg->grab_mode == 2 && p_events == 0) || (cfg->grab_mode == 3 && p_events > 0))) {
                     pulse_analyzer(&demod->fsk_pulse_data, package_type);
                 }
             } // if (package_type == ...
@@ -476,10 +547,10 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
                 || dumper->format == PULSE_OOK)
             continue;
         uint8_t *out_buf = iq_buf;  // Default is to dump IQ samples
-        unsigned long out_len = n_samples * 2 * demod->sample_size;
+        unsigned long out_len = n_samples * demod->sample_size;
 
         if (dumper->format == CU8_IQ) {
-            if (demod->sample_size == 2) {
+            if (demod->sample_size == 4) {
                 for (unsigned long n = 0; n < n_samples * 2; ++n)
                     ((uint8_t *)demod->buf.temp)[n] = (((int16_t *)iq_buf)[n] / 256) + 128; // scale Q0.15 to Q0.7
                 out_buf = (uint8_t *)demod->buf.temp;
@@ -487,7 +558,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
             }
         }
         else if (dumper->format == CS16_IQ) {
-            if (demod->sample_size == 1) {
+            if (demod->sample_size == 2) {
                 for (unsigned long n = 0; n < n_samples * 2; ++n)
                     ((int16_t *)demod->buf.temp)[n] = (iq_buf[n] * 256) - 32768; // scale Q0.7 to Q0.15
                 out_buf = (uint8_t *)demod->buf.temp; // this buffer is too small if out_block_size is large
@@ -495,11 +566,11 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
             }
         }
         else if (dumper->format == CS8_IQ) {
-            if (demod->sample_size == 1) {
+            if (demod->sample_size == 2) {
                 for (unsigned long n = 0; n < n_samples * 2; ++n)
                     ((int8_t *)demod->buf.temp)[n] = (iq_buf[n] - 128);
             }
-            else if (demod->sample_size == 2) {
+            else if (demod->sample_size == 4) {
                 for (unsigned long n = 0; n < n_samples * 2; ++n)
                     ((int8_t *)demod->buf.temp)[n] = ((int16_t *)iq_buf)[n] >> 8;
             }
@@ -507,11 +578,11 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
             out_len = n_samples * 2 * sizeof(int8_t);
         }
         else if (dumper->format == CF32_IQ) {
-            if (demod->sample_size == 1) {
+            if (demod->sample_size == 2) {
                 for (unsigned long n = 0; n < n_samples * 2; ++n)
                     ((float *)demod->buf.temp)[n] = (iq_buf[n] - 128) / 128.0f;
             }
-            else if (demod->sample_size == 2) {
+            else if (demod->sample_size == 4) {
                 for (unsigned long n = 0; n < n_samples * 2; ++n)
                     ((float *)demod->buf.temp)[n] = ((int16_t *)iq_buf)[n] / 32768.0f;
             }
@@ -539,7 +610,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
             out_len = n_samples * sizeof(float);
         }
         else if (dumper->format == F32_I) {
-            if (demod->sample_size == 1)
+            if (demod->sample_size == 2)
                 for (unsigned long n = 0; n < n_samples; ++n)
                     demod->f32_buf[n] = (iq_buf[n * 2] - 128) * (1.0f / 0x80); // scale from Q0.7
             else
@@ -549,7 +620,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
             out_len = n_samples * sizeof(float);
         }
         else if (dumper->format == F32_Q) {
-            if (demod->sample_size == 1)
+            if (demod->sample_size == 2)
                 for (unsigned long n = 0; n < n_samples; ++n)
                     demod->f32_buf[n] = (iq_buf[n * 2 + 1] - 128) * (1.0f / 0x80); // scale from Q0.7
             else
@@ -574,9 +645,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
         cfg->bytes_to_read -= len;
 
     if (cfg->after_successful_events_flag && (d_events > 0)) {
-#ifndef _WIN32
         alarm(0); // cancel the watchdog timer
-#endif
         if (cfg->after_successful_events_flag == 1) {
             cfg->exit_async = 1;
         }
@@ -591,15 +660,11 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
     int hop_index = cfg->hop_times > cfg->frequency_index ? cfg->frequency_index : cfg->hop_times - 1;
     if (cfg->hop_times > 0 && cfg->frequencies > 1
             && difftime(rawtime, cfg->hop_start_time) > cfg->hop_time[hop_index]) {
-#ifndef _WIN32
         alarm(0); // cancel the watchdog timer
-#endif
         cfg->hop_now = 1;
     }
     if (cfg->duration > 0 && rawtime >= cfg->stop_time) {
-#ifndef _WIN32
         alarm(0); // cancel the watchdog timer
-#endif
         cfg->exit_async = 1;
         fprintf(stderr, "Time expired, exiting!\n");
     }
@@ -615,9 +680,8 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
     if (cfg->hop_now && !cfg->exit_async) {
         cfg->hop_now = 0;
         time(&cfg->hop_start_time);
-        cfg->frequency_index  = (cfg->frequency_index + 1) % cfg->frequencies;
-        cfg->center_frequency = cfg->frequency[cfg->frequency_index];
-        sdr_set_center_freq(cfg->dev, cfg->center_frequency, 0);
+        cfg->frequency_index = (cfg->frequency_index + 1) % cfg->frequencies;
+        sdr_set_center_freq(cfg->dev, cfg->frequency[cfg->frequency_index], 0);
     }
 }
 
@@ -841,7 +905,7 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
             help_read();
 
         add_infile(cfg, arg);
-        // TODO: check_read_file_info()
+        // TODO: file_info_check_read()
         break;
     case 'w':
         if (!arg)
@@ -938,19 +1002,21 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
             cfg->report_protocol = 0;
         else if (!strcasecmp(arg, "level"))
             cfg->report_meta = 1;
+        else if (!strncasecmp(arg, "noise", 5))
+            cfg->report_noise = atoiv(arg_param(arg), 10); // atoi_time_default()
         else if (!strcasecmp(arg, "bits"))
             cfg->verbose_bits = 1;
         else if (!strcasecmp(arg, "description"))
             cfg->report_description = 1;
         else if (!strcasecmp(arg, "newmodel"))
-            cfg->old_model_keys = 0;
+            fprintf(stderr, "newmodel option (-M) is deprecated.\n");
         else if (!strcasecmp(arg, "oldmodel"))
-            cfg->old_model_keys = 1;
+            fprintf(stderr, "oldmodel option (-M) is deprecated.\n");
         else if (!strncasecmp(arg, "stats", 5)) {
             // there also should be options to set wether to flush on report
             char *p = arg_param(arg);
             cfg->report_stats = atoiv(p, 1);
-            cfg->stats_interval = atoiv(arg_param(p), 600);
+            cfg->stats_interval = atoiv(arg_param(p), 600); // atoi_time_default()
             time(&cfg->stats_time);
             cfg->stats_time += cfg->stats_interval;
         }
@@ -1020,7 +1086,7 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
             add_kv_output(cfg, arg_param(arg));
         }
         else if (strncmp(arg, "mqtt", 4) == 0) {
-            add_mqtt_output(cfg, arg_param(arg));
+            add_mqtt_output(cfg, arg);
         }
         else if (strncmp(arg, "influx", 6) == 0) {
             add_influx_output(cfg, arg);
@@ -1040,12 +1106,9 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
         }
         break;
     case 'K':
-        cfg->output_tag = arg;
-        cfg->output_key = asepc(&cfg->output_tag, '=');
-        if (!cfg->output_tag) {
-            cfg->output_tag = cfg->output_key;
-            cfg->output_key = "tag";
-        }
+        if (!arg)
+            help_tags();
+        add_data_tag(cfg, arg);
         break;
     case 'C':
         if (!arg)
@@ -1080,31 +1143,38 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
     case 'Y':
         if (!arg)
             usage(1);
-        char *p = arg;
+        char const *p = arg;
         while (p && *p) {
-            if (!strncmp(p, "auto", 4))
+            char const *val = NULL;
+            if (kwargs_match(p, "autolevel", &val))
+                cfg->demod->auto_level = atoiv(val, 1); // arg_float_default(p + 9, "-Y autolevel: ");
+            else if (kwargs_match(p, "squelch", &val))
+                cfg->demod->squelch_offset = atoiv(val, 1); // arg_float_default(p + 7, "-Y squelch: ");
+            else if (kwargs_match(p, "auto", &val))
                 cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_AUTO;
-            else if (!strncmp(p, "classic", 7))
+            else if (kwargs_match(p, "classic", &val))
                 cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_OLD;
-            else if (!strncmp(p, "minmax", 6))
+            else if (kwargs_match(p, "minmax", &val))
                 cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_NEW;
-            else if (!strncmp(p, "ampest", 6))
+            else if (kwargs_match(p, "ampest", &val))
                 cfg->demod->use_mag_est = 0;
-            else if (!strncmp(p, "verbose", 7))
+            else if (kwargs_match(p, "verbose", &val))
                 cfg->demod->detect_verbosity++;
-            else if (!strncmp(p, "magest", 6))
+            else if (kwargs_match(p, "magest", &val))
                 cfg->demod->use_mag_est = 1;
-            else if (!strncasecmp(p, "level", 5))
-                cfg->demod->level_limit = arg_float(p + 5, "-Y level: ");
-            else if (!strncasecmp(p, "minlevel", 8))
-                cfg->demod->min_level = arg_float(p + 8, "-Y minlevel: ");
-            else if (!strncasecmp(p, "minsnr", 6))
-                cfg->demod->min_snr = arg_float(p + 6, "-Y minsnr: ");
+            else if (kwargs_match(p, "level", &val))
+                cfg->demod->level_limit = arg_float(val, "-Y level: ");
+            else if (kwargs_match(p, "minlevel", &val))
+                cfg->demod->min_level = arg_float(val, "-Y minlevel: ");
+            else if (kwargs_match(p, "minsnr", &val))
+                cfg->demod->min_snr = arg_float(val, "-Y minsnr: ");
+            else if (kwargs_match(p, "filter", &val))
+                cfg->demod->low_pass = arg_float(val, "-Y filter: ");
             else {
                 fprintf(stderr, "Unknown pulse detector setting: %s\n", p);
                 usage(1);
             }
-            p = arg_param(p);
+            p = kwargs_skip(p);
         }
         break;
     case 'E':
@@ -1133,11 +1203,12 @@ static r_cfg_t g_cfg;
 
 #ifdef _WIN32
 BOOL WINAPI
-sighandler(int signum)
+console_handler(int signum)
 {
     if (CTRL_C_EVENT == signum) {
         fprintf(stderr, "Signal caught, exiting!\n");
         g_cfg.exit_async = 1;
+        sdr_stop(g_cfg.dev);
         return TRUE;
     }
     else if (CTRL_BREAK_EVENT == signum) {
@@ -1145,8 +1216,23 @@ sighandler(int signum)
         g_cfg.hop_now = 1;
         return TRUE;
     }
+    else if (signum == SIGALRM) {
+        fprintf(stderr, "Async read stalled, exiting!\n");
+        g_cfg.exit_code = 3;
+        g_cfg.exit_async = 1;
+        sdr_stop(g_cfg.dev);
+        return TRUE;
+    }
     return FALSE;
 }
+
+/* Only called for SIGALRM
+ */
+static void sighandler(int signum)
+{
+  console_handler(signum);
+}
+
 #else
 static void sighandler(int signum)
 {
@@ -1169,6 +1255,7 @@ static void sighandler(int signum)
         fprintf(stderr, "Signal caught, exiting!\n");
     }
     g_cfg.exit_async = 1;
+    sdr_stop(g_cfg.dev);
 }
 #endif
 
@@ -1178,16 +1265,19 @@ static void sdr_handler(sdr_event_t *ev, void *ctx)
 
     data_t *data = NULL;
     if (ev->ev & SDR_EV_RATE) {
+        cfg->samp_rate = ev->sample_rate;
         data = data_append(data,
                 "sample_rate", "", DATA_INT, ev->sample_rate,
                 NULL);
     }
     if (ev->ev & SDR_EV_CORR) {
+        cfg->ppm_error = ev->freq_correction;
         data = data_append(data,
                 "freq_correction", "", DATA_INT, ev->freq_correction,
                 NULL);
     }
     if (ev->ev & SDR_EV_FREQ) {
+        cfg->center_frequency = ev->center_frequency;
         data = data_append(data,
                 "center_frequency", "", DATA_INT, ev->center_frequency,
                 "frequencies", "", DATA_COND, cfg->frequencies > 1, DATA_ARRAY, data_array(cfg->frequencies, DATA_INT, cfg->frequency),
@@ -1208,7 +1298,8 @@ static void sdr_handler(sdr_event_t *ev, void *ctx)
 
     if (ev->ev == SDR_EV_DATA) {
         if (cfg->mgr) {
-            mg_mgr_poll(cfg->mgr, 0);
+            int max_polls = 16;
+            while (max_polls-- && mg_mgr_poll(cfg->mgr, 0));
         }
 
         if (!cfg->exit_async)
@@ -1225,7 +1316,6 @@ int main(int argc, char **argv) {
 #endif
     FILE *in_file;
     int r = 0;
-    unsigned i;
     struct dm_state *demod;
     r_cfg_t *cfg = &g_cfg;
 
@@ -1238,35 +1328,23 @@ int main(int argc, char **argv) {
 
     demod = cfg->demod;
 
-    demod->pulse_detect = pulse_detect_create();
-
-    /* initialize tables */
-    baseband_init();
-
-    r_device r_devices[] = {
-#define DECL(name) name,
-            DEVICES
-#undef DECL
-            };
-
-    cfg->num_r_devices = sizeof(r_devices) / sizeof(*r_devices);
-    for (i = 0; i < cfg->num_r_devices; i++) {
-        r_devices[i].protocol_num = i + 1;
-    }
-    cfg->devices = r_devices;
-
     // if there is no explicit conf file option look for default conf files
     if (!hasopt('c', argc, argv, OPTSTRING)) {
         parse_conf_try_default_files(cfg);
     }
 
     parse_conf_args(cfg, argc, argv);
-
-    // warn if still using old model keys
-    if (cfg->old_model_keys) {
-        fprintf(stderr,
-                "\n\tWarning: Using deprecated old model keys (\"-M oldmodel\"). This will be removed shortly.\n\n");
+    // apply hop defaults and set first frequency
+    if (cfg->frequencies == 0) {
+        cfg->frequency[0] = DEFAULT_FREQUENCY;
+        cfg->frequencies  = 1;
     }
+    cfg->center_frequency = cfg->frequency[cfg->frequency_index];
+    if (cfg->frequencies > 1 && cfg->hop_times == 0) {
+        cfg->hop_time[cfg->hop_times++] = DEFAULT_HOP_TIME;
+    }
+    // save sample rate, this should be a hop config too
+    uint32_t sample_rate_0 = cfg->samp_rate;
 
     // add all remaining positional arguments as input files
     while (argc > optind) {
@@ -1323,30 +1401,36 @@ int main(int argc, char **argv) {
         }
     }
 
-    fprintf(stderr, "Registered %zu out of %d device decoding protocols",
-            demod->r_devs.len, cfg->num_r_devices);
-
-    if (!cfg->verbosity) {
-        // print registered decoder ranges
-        fprintf(stderr, " [");
-        for (void **iter = demod->r_devs.elems; iter && *iter; ++iter) {
-            r_device *r_dev = *iter;
-            unsigned num = r_dev->protocol_num;
-            if (num == 0)
-                continue;
-            while (iter[1]
-                    && r_dev->protocol_num + 1 == ((r_device *)iter[1])->protocol_num)
-                r_dev = *++iter;
-            if (num == r_dev->protocol_num)
-                fprintf(stderr, " %u", num);
-            else
-                fprintf(stderr, " %u-%u", num, r_dev->protocol_num);
+    {
+        char decoders_str[1024];
+        decoders_str[0] = '\0';
+        if (!cfg->verbosity) {
+            abuf_t p = {0};
+            abuf_init(&p, decoders_str, sizeof(decoders_str));
+            // print registered decoder ranges
+            abuf_printf(&p, " [");
+            for (void **iter = demod->r_devs.elems; iter && *iter; ++iter) {
+                r_device *r_dev = *iter;
+                unsigned num = r_dev->protocol_num;
+                if (num == 0)
+                    continue;
+                while (iter[1]
+                        && r_dev->protocol_num + 1 == ((r_device *)iter[1])->protocol_num)
+                    r_dev = *++iter;
+                if (num == r_dev->protocol_num)
+                    abuf_printf(&p, " %u", num);
+                else
+                    abuf_printf(&p, " %u-%u", num, r_dev->protocol_num);
+            }
+            abuf_printf(&p, " ]");
         }
-        fprintf(stderr, " ]");
+        fprintf(stderr, "Registered %zu out of %u device decoding protocols%s\n",
+                demod->r_devs.len, cfg->num_r_devices, decoders_str);
     }
-    fprintf(stderr, "\n");
 
-    start_outputs(cfg, well_known_output_fields(cfg));
+    char const **well_known = well_known_output_fields(cfg);
+    start_outputs(cfg, well_known);
+    free(well_known);
 
     if (cfg->out_block_size < MINIMAL_BUF_LENGTH ||
             cfg->out_block_size > MAXIMAL_BUF_LENGTH) {
@@ -1479,8 +1563,13 @@ int main(int argc, char **argv) {
         for (void **iter = cfg->in_files.elems; iter && *iter; ++iter) {
             cfg->in_filename = *iter;
 
-            parse_file_info(cfg->in_filename, &demod->load_info);
-            if (strcmp(demod->load_info.path, "-") == 0) { /* read samples from stdin */
+            file_info_clear(&demod->load_info); // reset all info
+            file_info_parse_filename(&demod->load_info, cfg->in_filename);
+            // apply file info or default
+            cfg->samp_rate        = demod->load_info.sample_rate ? demod->load_info.sample_rate : sample_rate_0;
+            cfg->center_frequency = demod->load_info.center_frequency ? demod->load_info.center_frequency : cfg->frequency[0];
+
+            if (strcmp(demod->load_info.path, "-") == 0) { // read samples from stdin
                 in_file = stdin;
                 cfg->in_filename = "<stdin>";
             } else {
@@ -1492,12 +1581,13 @@ int main(int argc, char **argv) {
             }
             fprintf(stderr, "Test mode active. Reading samples from file: %s\n", cfg->in_filename);  // Essential information (not quiet)
             if (demod->load_info.format == CU8_IQ
+                    || demod->load_info.format == CS8_IQ
                     || demod->load_info.format == S16_AM
                     || demod->load_info.format == S16_FM) {
-                demod->sample_size = sizeof(uint8_t); // CU8, AM, FM
+                demod->sample_size = sizeof(uint8_t) * 2; // CU8, AM, FM
             } else if (demod->load_info.format == CS16_IQ
                     || demod->load_info.format == CF32_IQ) {
-                demod->sample_size = sizeof(int16_t); // CF32, CS16
+                demod->sample_size = sizeof(int16_t) * 2; // CS16, CF32 (after conversion)
             } else if (demod->load_info.format == PULSE_OOK) {
                 // ignore
             } else {
@@ -1551,6 +1641,7 @@ int main(int argc, char **argv) {
             int n_blocks = 0;
             unsigned long n_read;
             do {
+                // Convert CF32 file to CS16 buffer
                 if (demod->load_info.format == CF32_IQ) {
                     n_read = fread(test_mode_float_buf, sizeof(float), DEFAULT_BUF_LENGTH / 2, in_file);
                     // clamp float to [-1,1] and scale to Q0.15
@@ -1565,15 +1656,22 @@ int main(int argc, char **argv) {
                     n_read *= 2; // convert to byte count
                 } else {
                     n_read = fread(test_mode_buf, 1, DEFAULT_BUF_LENGTH, in_file);
+
+                    // Convert CS8 file to CU8 buffer
+                    if (demod->load_info.format == CS8_IQ) {
+                        for (unsigned long n = 0; n < n_read; n++) {
+                            test_mode_buf[n] = ((int8_t)test_mode_buf[n]) + 128;
+                        }
+                    }
                 }
                 if (n_read == 0) break;  // sdr_callback() will Segmentation Fault with len=0
-                demod->sample_file_pos = ((float)n_blocks * DEFAULT_BUF_LENGTH + n_read) / cfg->samp_rate / 2 / demod->sample_size;
+                demod->sample_file_pos = ((float)n_blocks * DEFAULT_BUF_LENGTH + n_read) / cfg->samp_rate / demod->sample_size;
                 n_blocks++; // this assumes n_read == DEFAULT_BUF_LENGTH
                 sdr_callback(test_mode_buf, n_read, cfg);
             } while (n_read != 0 && !cfg->exit_async);
 
             // Call a last time with cleared samples to ensure EOP detection
-            if (demod->sample_size == 1) { // CU8
+            if (demod->sample_size == 2) { // CU8
                 memset(test_mode_buf, 128, DEFAULT_BUF_LENGTH); // 128 is 0 in unsigned data
                 // or is 127.5 a better 0 in cu8 data?
                 //for (unsigned long n = 0; n < DEFAULT_BUF_LENGTH/2; n++)
@@ -1582,11 +1680,9 @@ int main(int argc, char **argv) {
             else { // CF32, CS16
                     memset(test_mode_buf, 0, DEFAULT_BUF_LENGTH);
             }
-            demod->sample_file_pos = ((float)n_blocks + 1) * DEFAULT_BUF_LENGTH / cfg->samp_rate / 2 / demod->sample_size;
+            demod->sample_file_pos = ((float)n_blocks + 1) * DEFAULT_BUF_LENGTH / cfg->samp_rate / demod->sample_size;
             sdr_callback(test_mode_buf, DEFAULT_BUF_LENGTH, cfg);
-#ifndef _WIN32
             alarm(0); // cancel the watchdog timer
-#endif
 
             //Always classify a signal at the end of the file
             if (demod->am_analyze)
@@ -1612,11 +1708,13 @@ int main(int argc, char **argv) {
     }
 
     // Normal case, no test data, no in files
-    r = sdr_open(&cfg->dev, &demod->sample_size, cfg->dev_query, cfg->verbosity);
+    r = sdr_open(&cfg->dev, cfg->dev_query, cfg->verbosity);
     if (r < 0) {
         exit(2);
     }
     cfg->dev_info = sdr_get_dev_info(cfg->dev);
+    demod->sample_size = sdr_get_sample_size(cfg->dev);
+    //demod->sample_signed = sdr_get_sample_signed(cfg->dev);
 
 #ifndef _WIN32
     sigact.sa_handler = sighandler;
@@ -1629,7 +1727,7 @@ int main(int argc, char **argv) {
     sigaction(SIGUSR1, &sigact, NULL);
     sigaction(SIGINFO, &sigact, NULL);
 #else
-    SetConsoleCtrlHandler((PHANDLER_ROUTINE)sighandler, TRUE);
+    SetConsoleCtrlHandler((PHANDLER_ROUTINE)console_handler, TRUE);
 #endif
     /* Set the sample rate */
     r = sdr_set_sample_rate(cfg->dev, cfg->samp_rate, 1); // always verbose
@@ -1651,13 +1749,6 @@ int main(int argc, char **argv) {
         fprintf(stderr, "WARNING: Failed to reset buffers.\n");
     r = sdr_activate(cfg->dev);
 
-    if (cfg->frequencies == 0) {
-        cfg->frequency[0] = DEFAULT_FREQUENCY;
-        cfg->frequencies = 1;
-    }
-    if (cfg->frequencies > 1 && cfg->hop_times == 0) {
-        cfg->hop_time[cfg->hop_times++] = DEFAULT_HOP_TIME;
-    }
     if (cfg->verbosity) {
         fprintf(stderr, "Reading samples in async mode...\n");
     }
@@ -1666,22 +1757,19 @@ int main(int argc, char **argv) {
         cfg->stop_time += cfg->duration;
     }
 
-    cfg->center_frequency = cfg->frequency[cfg->frequency_index];
     r = sdr_set_center_freq(cfg->dev, cfg->center_frequency, 1); // always verbose
 
         time(&cfg->hop_start_time);
-#ifndef _WIN32
         signal(SIGALRM, sighandler);
         alarm(3); // require callback to run every 3 second, abort otherwise
-#endif
+
         r = sdr_start(cfg->dev, sdr_handler, (void *)cfg,
                 DEFAULT_ASYNC_BUF_NUMBER, cfg->out_block_size);
         if (r < 0) {
             fprintf(stderr, "WARNING: async read failed (%i).\n", r);
         }
-#ifndef _WIN32
+
         alarm(0); // cancel the watchdog timer
-#endif
 
     if (cfg->report_stats > 0) {
         event_occurred_handler(cfg, create_report_data(cfg, cfg->report_stats));
