@@ -97,7 +97,7 @@ void samp_grab_write(samp_grab_t *g, unsigned grab_len, unsigned grab_end)
     char f_name[64] = {0};
     FILE *fp;
 
-    char *format = *g->sample_size == 1 ? "cu8" : "cs16";
+    char *format = *g->sample_size == 2 ? "cu8" : "cs16";
     double freq_mhz = *g->frequency / 1000000.0;
     double rate_khz = *g->samp_rate / 1000.0;
     while (1) {
@@ -108,7 +108,7 @@ void samp_grab_write(samp_grab_t *g, unsigned grab_len, unsigned grab_end)
         }
     }
 
-    signal_bsize = *g->sample_size * 2 * grab_len;
+    signal_bsize = *g->sample_size * grab_len;
     signal_bsize += BLOCK_SIZE - (signal_bsize % BLOCK_SIZE);
 
     if (signal_bsize > g->sg_len) {
@@ -117,7 +117,7 @@ void samp_grab_write(samp_grab_t *g, unsigned grab_len, unsigned grab_end)
     }
 
     // relative end in bytes from current sg_index down
-    end_pos = *g->sample_size * 2 * grab_end;
+    end_pos = *g->sample_size * grab_end;
     if (g->sg_index >= end_pos)
         end_pos = g->sg_index - end_pos;
     else
